/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * Author : Alexandre Parenteau <aubonbeurre@geocities.com> --- December 1997
 */

/*
 * AppGlue.cpp --- glue code to access CVS shared library
 */

#include "stdafx.h"

#include "TextBinary.h"
#	include <process.h>
#	include <direct.h>
#include <string.h>

#	ifdef _DEBUG
#	define new DEBUG_NEW
#	undef THIS_FILE
	static char THIS_FILE[] = __FILE__;
#	endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#ifdef HAVE_ERRNO_H
#	include <errno.h>
#endif
#include <vector>

#include "Authen.h"
#include "CvsPrefs.h"
#include "AppGlue.h"
#include "AppConsole.h"
#include "CvsArgs.h"

#include "dll_process.h"
#include "dll_loader.h"

using namespace std;
static bool first=true;
static CRITICAL_SECTION lock;

#define errInternal -99
#define errThreadRunning -98

//int launchCVS(bool wincvslib, const char *path, int argc, char * const *argv,
//	CCvsConsole * console = 0L);

// overide console
static CCvsConsole * sConsole = 0L;

#ifdef WIN32
	static CWinThread *sLastWin32CvsThread = 0L;
#endif

extern "C" {
	static long consoleout(char *txt, long len);
	static long consoleerr(char *txt, long len);
	static long consolein(char *txt, long len);
	static char *mygetenv(char *name);
}

void cvs_err(const char *format, ...)
{
	va_list args;
	char mess[1024] = {'\0'};

	va_start (args, format);
	vsprintf (mess, format, args);
	va_end (args);
    if (sConsole!=0)
    {
        sConsole->Err(mess, strlen(mess));
    }

#ifdef WIN32
    // TODO: redo
/*	CWinApp* app = AfxGetApp();
	//ASSERT(app->IsKindOf(RUNTIME_CLASS(CWincvsApp)));
	CWincvsView *view = ((CWincvsApp *)app)->GetConsoleView();
	if(view == NULL)
		return;

	view->OutConsole(mess, strlen(mess), true);*/
#endif /* WIN32 */
}

void cvs_out(const char *format, ...)
{
	va_list args;
	char mess[1024] = {'\0'};

	va_start (args, format);
	vsprintf (mess, format, args);
	va_end (args);
    if (sConsole!=0)
    {
        sConsole->Out(mess, strlen(mess));
    }

#ifdef WIN32
    // TODO: redo
/*	CWinApp* app = AfxGetApp();
	//ASSERT(app->IsKindOf(RUNTIME_CLASS(CWincvsApp)));
	CWincvsView *view = ((CWincvsApp *)app)->GetConsoleView();
	if(view == NULL)
		return;

	view->OutConsole(mess, strlen(mess), true);*/
#endif /* WIN32 */
}

void cvs_outstr(const char *str, size_t len)
{
#ifdef WIN32
/*	CWinApp* app = AfxGetApp();
	//ASSERT(app->IsKindOf(RUNTIME_CLASS(CWincvsApp)));
	CWincvsView *view = ((CWincvsApp *)app)->GetConsoleView();
	if(view == NULL)
		return;

	view->OutConsole(str, len);*/
#endif /* WIN32 */
}

void cvs_errstr(const char *str, size_t len)
{
#ifdef WIN32
/*	CWinApp* app = AfxGetApp();
	//ASSERT(app->IsKindOf(RUNTIME_CLASS(CWincvsApp)));
	CWincvsView *view = ((CWincvsApp *)app)->GetConsoleView();
	if(view == NULL)
		return;

	view->OutConsole(str, len, true);*/
#endif /* WIN32 */
}

#ifdef _MSC_VER
extern "C"
#endif
static long consoleout(char *txt, long len)
{
	if(sConsole != 0L)
		return sConsole->Out(txt, len);
	return 0;

}

#ifdef _MSC_VER
extern "C"
#endif
static long consoleerr(char *txt, long len)
{
	if(sConsole != 0L)
		return sConsole->Err(txt, len);
	return 0;

}

#ifdef _MSC_VER
extern "C"
#endif
static long consolein(char *txt, long len)
{
	if(sConsole != 0L)
		return sConsole->In(txt, len);
	return 0;
}

/* NT has two evironment variables, HOMEPATH and HOMEDRIVE, which,
   when combined as ${HOMEDRIVE}${HOMEPATH}, give the unix equivalent
   of HOME.  Some NT users are just too unixy, though, and set the
   HOME variable themselves.  Therefore, we check for HOME first, and
   then try to combine the other two if that fails.

   Looking for HOME strikes me as bogus, particularly if the only reason
   is to cater to "unixy users".  On the other hand, if the reasoning is
   there should be a single variable, rather than requiring people to
   set both HOMEDRIVE and HOMEPATH, then it starts to make a little more
   sense.

   Win95: The system doesn't set HOME, HOMEDRIVE, or HOMEPATH (at
   least if you set it up as the "all users under one user ID" or
   whatever the name of that option is).  Based on thing overheard on
   the net, it seems that users of the pserver client have gotten in
   the habit of setting HOME (if you don't use pserver, you can
   probably get away without having a reasonable return from
   get_homedir.  Of course you lose .cvsrc and .cvsignore, but many
   users won't notice).  So it would seem that we should be somewhat
   careful if we try to change the current behavior.

   NT 3.51 or NT 4.0: I haven't checked this myself, but I am told
   that HOME gets set, but not to the user's home directory.  It is
   said to be set to c:\users\default by default.  */

char *
get_homedir ()
{
    static char *pathbuf;
    char *hd, *hp;

    if (pathbuf != NULL)
	return pathbuf;
    else if ((hd = getenv ("HOME")))
	return hd;
    else if ((hd = getenv ("HOMEDRIVE")) && (hp = getenv ("HOMEPATH")))
    {
	pathbuf = new char[strlen (hd) + strlen (hp) + 5];
	strcpy (pathbuf, hd);
	strcat (pathbuf, hp);

#ifdef _MSC_VER
	/*
	 * the visual c++ c-library won't open anymore "C:\/cvsrc"
	 * so we have to remove the training '\'
	 */
	if(pathbuf[strlen(pathbuf)-1] == '\\')
	{
		pathbuf[strlen(pathbuf)-1] = '\0';
	}
#endif

	return pathbuf;
    }
    else
	return NULL;
}

#ifdef _MSC_VER
extern "C"
#endif
static char *mygetenv(char *name)
{
	if (strcmp(name, "CVS_GETPASS")==0)
	{
		if (sConsole) return sConsole->GetPassword();
		return "";
	}
	if(strcmp(name, "CVSLIB_YESNO") == 0)
	{
		if (sConsole)
		{
			if (sConsole->YesNo())
			{
				return "yes";
			}
			else
			{
				return "no";
			}
		}
		return "No";
	}
	if (strcmp(name, "HOME")==0) return get_homedir();
    if(strcmp(name, "DLLLIBSTOP") == 0)
    {
        if (sConsole && sConsole->ShouldStop())
        {
            return (char *)-1;
        }
        else
        {
            return 0;
        } 
    }
    if (1) return getenv(name);
	// synchonize it since the thread calls and
	// we call some MFC objects in it (prompt
	// password, get path...)
//	CCvsThreadLock();

	static char port[10];

	if(name == NULL)
		return NULL;
		
	
	if(strcmp(name, "USER") == 0
		// I do that (and see the "STRATA_HACK" in
		// subr.c) because people get tired on Windows
		// to have to check their Windows login :
		// this is only for .rhosts authentication.
		|| strcmp(name, "LOGNAME") == 0
		)
	{
		static char user[64];
		
		// extract from the cvsroot
		const char *ccvsroot = gCvsPrefs;
		ccvsroot = Authen::skiptoken(ccvsroot);
		char *login = strchr(ccvsroot, '@');
		if(login == NULL)
		{
			// for WIN32 this means the CVSROOT is local
			return NULL;
		}
		int length = login - ccvsroot;
		memcpy(user, ccvsroot, length * sizeof(char));
		user[length] = '\0';
		return user;
	}
	else if(strcmp(name, "CVSROOT") == 0)
	{
		static CStr root;
		
		// extract from the cvsroot
		const char *ccvsroot = gCvsPrefs;
		ccvsroot = Authen::skiptoken(ccvsroot);
		root = gAuthen.token();
		root << ccvsroot;
		return root;
	}
	else if(strcmp(name, "CVSREAD") == 0)
	{
		return gCvsPrefs.CheckoutRO() ? "1" : 0L;
	}
	else if(strcmp(name, "CVS_GETPASS") == 0)
	{
//		return CompatGetPassword();
	}
	else if(strcmp(name, "CVS_PSERVER_PORT") == 0)
	{
		if(gCvsPrefs.PserverPort() != -1)
		{
			sprintf(port, "%d", gCvsPrefs.PserverPort());
			return port;
		}
		
		return 0L;
	}
	else if(strcmp(name, "CVS_RCMD_PORT") == 0)
	{
		if(gCvsPrefs.RhostPort() != -1)
		{
			sprintf(port, "%d", gCvsPrefs.RhostPort());
			return port;
		}
		
		return 0L;
	}
	else if(strcmp(name, "CVS_SERVER") == 0)
	{
		return (char *)gCvsPrefs.ServerName();
	}
	else if(strcmp(name, "CVS_RSH") == 0)
	{
		if(gAuthen.kind() == ssh && gCvsPrefs.RshName() == 0L)
			return "ssh";
		return (char *)gCvsPrefs.RshName();
	}
	else if(strcmp(name, "CVSLIB_YESNO") == 0)
	{
//		return (char *)AskYesNo();
	}
	else if(strcmp(name, "DIRTY_MCP") == 0)
	{
		return gCvsPrefs.DirtySupport() ? "yes" : "no";
	}
	else if(strcmp(name, "HOME") == 0)
	{
		// prompt for home the first time
		if(gCvsPrefs.Home() == 0L)
		{
			char oldpath[512];
			getcwd(oldpath, 511);
			const char *chome = 0L;//BrowserGetDirectory("Select your home directory :");
			if(chome == 0L)
			{
				chdir(oldpath);
				return 0L;
			}

			gCvsPrefs.SetHome(chome);
			gCvsPrefs.save();
			chdir(oldpath);
		}

		// Windows 98 doesn't like C:\/.cvspass
		static CStr home;
		home = gCvsPrefs.Home();
		if(home.endsWith(kPathDelimiter))
		{
			home[home.length() - 1] = '\0';
		}

		return home;
	}
	else if(strcmp(name, "DLLLIBSTOP") == 0)
	{
//		CWincvsApp* app = (CWincvsApp *)AfxGetApp();
//		return app->gCvsStopping ? (char *)-1 : 0L;
	}
	
	return NULL;
}


static bool loadCVS(bool useWinCvslib, CompatConnectID & connID)
{
#ifdef WIN32
// TODO:renew
    /*CWincvsApp* app = (CWincvsApp *)AfxGetApp();
	if(app->gCvsRunning)
	{
		cvs_err("Error : cvs is already running !\n");
		return false;
	}*/
#endif /* WIN32 */

	int result = 0;
	const char *name_dll2;
    HKEY hkey;
    RegOpenKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\Jalindi\\Igloo", 0, KEY_QUERY_VALUE, &hkey);
	
	// get cvs shared library
    if (useWinCvslib) // (getenv("CVSROOT")!=NULL && _strnicmp(getenv("CVSROOT"), ":ntserver", 9)==0)
    {
	    name_dll2 = "cvs2ntlib.dll";
    }
    else
    {
	    name_dll2 = "cvs2ntslib.dll";
    }
    LPCTSTR lpValueName="SccServerPath";
    DWORD type;
    BYTE data[1000];
    DWORD length=1000;
    RegQueryValueEx(hkey, lpValueName, NULL, &type, data, &length);
    RegCloseKey(hkey);
    char* fulldll=new char[length+strlen(name_dll2)];
    strncpy(fulldll, (char*)data, length-10);
    strcpy(fulldll+length-10, name_dll2);

    // open the library
    if(!CompatLoadLibrary(&connID, fulldll))
    {
        char errorBuffer[2000];
        errorBuffer[0]=0;
        DWORD result= FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,      // source and processing options
            NULL,   // message source
            GetLastError(),  // message identifier
            0, // language identifier
            errorBuffer,    // message buffer
            2000,        // maximum size of message buffer
            NULL  // array of message inserts
            );
        cvs_out("Error: Unable to load the library %s\n", fulldll);
        cvs_out("Error: %s\n", errorBuffer);
        result=GetFileAttributes(fulldll);
        if (result<0) 
        {
            cvs_out("%s does not exist\n", fulldll);
        }
        else
        {
            cvs_out("%s does exist\n", fulldll);
        }
		delete [] fulldll;
        return false;
    }
    delete [] fulldll;

	// set the console out
	if(!CompatCallLibrary1(connID, result, uppSetGlueProcInfo, "dllglue_setconsoleout",
		dllglue_setglue_func, CompatFuncArg(uppConsoleOutProcInfo, consoleout)))
	{
		cvs_err("Error : Cannot find symbol ""dllglue_setconsoleout""\n");
		goto closeandfail;
	}

	// set the console err
	if(!CompatCallLibrary1(connID, result, uppSetGlueProcInfo, "dllglue_setconsoleerr",
		dllglue_setglue_func, CompatFuncArg(uppConsoleOutProcInfo, consoleerr)))
	{
		cvs_err("Error : Cannot find symbol ""dllglue_setconsoleerr""\n");
		goto closeandfail;
	}

	// set the console in
	if(!CompatCallLibrary1(connID, result, uppSetGlueProcInfo, "dllglue_setconsolein",
		dllglue_setglue_func, CompatFuncArg(uppConsoleInProcInfo, consolein)))
	{
		cvs_err("Error : Cannot find symbol ""dllglue_setconsolein""\n");
		goto closeandfail;
	}

	// set the getenv
	if(!CompatCallLibrary1(connID, result, uppSetGlueProcInfo, "dllglue_setgetenv",
		dllglue_setglue_func, CompatFuncArg(uppGetEnvProcInfo, mygetenv)))
	{
		cvs_err("Error : Cannot find symbol ""dllglue_setgetenv""\n");
		goto closeandfail;
	}

		
	return true;
closeandfail:
	CompatCloseLibrary(&connID);
	return false;
}

static void unloadCVS(CompatConnectID connID, int exitc)
{	
	CompatCloseLibrary(&connID);
	
//	cvs_out("\n*****CVS exited normally with code %d*****\n\n", exitc);

#ifdef WIN32
/*	CWincvsApp* app = (CWincvsApp *)AfxGetApp();
	if(app->gCvsStopping)
	{
		cvs_out("*****CVS stopped on user request !!! *****\n\n");
	}*/
#endif /* WIN32 */
}

#ifdef WIN323
// critical section to protect while accessing MFC from the DLL
CRITICAL_SECTION CCvsThread::m_lock;

CCvsThread::CCvsThread()
{
}

void CCvsThread::Delete()
{
	// calling the base here won't do anything but it is a good habit
	CWinThread::Delete();
}

void CCvsThread::KillThread()
{
//	CWincvsApp* app = (CWincvsApp *)AfxGetApp();
//	app->gCvsStopping = true;
}

CCvsThread::~CCvsThread()
{
	sLastWin32CvsThread = 0L;
}

BOOL CCvsThread::InitInstance()
{
    // TODO: redo
//	CWincvsApp* app = (CWincvsApp *)AfxGetApp();

	if(!loadCVS(mConnID))
		return FALSE;
	
//	app->gCvsRunning = true;
//	app->gCvsStopping = false;

//	app->gLastCvsResult = Launch();

	unloadCVS(mConnID, true);//app->gLastCvsResult);

//	app->gCvsRunning = false;
//	app->gCvsStopping = false;

//	app->GetFileView()->CheckChanges();

	return FALSE;
}

IMPLEMENT_DYNAMIC(CCvsThread, CWinThread);

BEGIN_MESSAGE_MAP(CCvsThread, CWinThread)
	//{{AFX_MSG_MAP(CCvsThread)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

class CCvsCmdThread : public CCvsThread
{
public:
	DECLARE_DYNAMIC(CCvsCmdThread)

	CCvsCmdThread(const char *path, int argc, char * const *argv);
	virtual ~CCvsCmdThread();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCvsCmdThread)
	//}}AFX_VIRTUAL

protected:
	virtual int Launch();

	CvsArgs mArgs;
	CPStr mPath;

	// Generated message map functions
	//{{AFX_MSG(CCvsCmdThread)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

CCvsCmdThread::CCvsCmdThread(const char *path, int argc, char * const *argv) :
		mArgs(argv, argc), mPath(path), CCvsThread()
{
}

CCvsCmdThread::~CCvsCmdThread()
{
}

int CCvsCmdThread::Launch()
{
	int exitc = 0;

	if(!CompatCallLibrary3(mConnID, exitc, uppMainProcInfo, "dllglue_main",
		dllglue_main_func, mPath.empty() ? 0L : (const char *)mPath,
		mArgs.Argc(), (char **)mArgs.Argv()))
	{
		cvs_err("Error : Cannot find symbol ""dllglue_main""\n");
		exitc = 1;
	}
	return exitc;
}

IMPLEMENT_DYNAMIC(CCvsCmdThread, CCvsThread);

BEGIN_MESSAGE_MAP(CCvsCmdThread, CCvsThread)
	//{{AFX_MSG_MAP(CCvsCmdThread)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

#endif /* WIN32 */

int launchCVS(bool wincvslib, const char *path, int argc, char * const *argv, CCvsConsole * console)
{
    if (first)
    {
        InitializeCriticalSection(&lock);
        first=false;
    }
    EnterCriticalSection(&lock);
	sConsole = console;
	
	// sorry for Windows 98 and Windows 95,
	// it takes too much time to launch the thread...
//	if(/*gCvsPrefs.WhichVersion() == kWin32 || */console != 0L)
	{
	sLastWin32CvsThread = 0L;
	CompatConnectID connID;
	
	if(!loadCVS(wincvslib, connID))
		return errInternal;
	
	int exitc = 0;

	// call the main
	if(!CompatCallLibrary3(connID, exitc, uppMainProcInfo, "dllglue_main",
		dllglue_main_func, path, argc, (char **)argv))
	{
		cvs_err("Error : Cannot find symbol ""dllglue_main""\n");
		exitc = errInternal;
	}

	unloadCVS(connID, exitc);
	console->Flush();
	sConsole=0;
    LeaveCriticalSection(&lock);
	return exitc;
    }
#ifdef WIN323
	else
	{
	CCvsCmdThread* pThread;
	pThread = new CCvsCmdThread(path, argc, (char **)argv);
	if (pThread == 0L)
	{
		cvs_err("Impossible to create the cvs thread !\n");
		return errInternal;
	}

	ASSERT_VALID(pThread);

	// Create Thread in a suspended state so we can set the Priority 
	// before it starts getting away from us
	if (!pThread->CreateThread(CREATE_SUSPENDED))
	{
		cvs_err("Impossible to start the cvs thread !\n");
		delete pThread;
		return errInternal;
	}

	// If you want to make the sample more sprightly, set the thread priority here 
	// a little higher. It has been set at idle priority to keep from bogging down 
	// other apps that may also be running.
	VERIFY(pThread->SetThreadPriority(THREAD_PRIORITY_NORMAL));
	// Now the thread can run wild
	pThread->ResumeThread();
	sLastWin32CvsThread = pThread;

	return errThreadRunning;
	}
#endif /* WIN32 */
}

class CStreamConsole : public CCvsConsole
{
	bool loose;
public:
	CStreamConsole(ostream *stream) : outStream(*stream), loose(false) {}

	virtual long cvs_out(char *txt, long len)
	{
        char *s=new char[len+1];
        strncpy(s, txt, len);
        s[len]=0;
        outStream << s;
		loose = s[len-1]!='\n';
//		fwrite(txt, sizeof(char), len, fOut);
		delete [] s;
		return len;
	}
	virtual long cvs_err(char *txt, long len)
	{
        char *s=new char[len+1];
        strncpy(s, txt, len);
        s[len]=0;
        if (len==0) return len;
        if (len>=7 && strncmp("Garbage", txt, 7)==0) return len;
        if (len>=8 && strncmp("Garbage", txt+1, 7)==0) return len;
        if (len>=9 && strncmp("Garbage", txt+2, 7)==0) return len;
        outStream << s;
		loose = s[len-1]!='\n';
		delete [] s;
//		cvs_errstr(txt, len);
		return len;
	}
	virtual void flush()
	{
		if (loose) outStream << endl;
		loose = false;
	}
protected:
	ostream &outStream;
};

class CNullConsole : public CCvsConsole
{
public:
	CNullConsole() {}

	virtual long cvs_out(char *txt, long len)
	{
		return len;
	}
	virtual long cvs_err(char *txt, long len)
	{
		return len;
	}
	virtual void flush()
	{

	}
};

class CTmpFiles
{
public:
	~CTmpFiles()
	{
		std::vector<CStr>::const_iterator i;
		for(i = allfiles.begin(); i != allfiles.end(); ++i)
		{
			const char *file = *i;
			unlink((char *)file);
		}
	}

	inline const char *push_back(const char *file)
	{
		allfiles.push_back(file);
		return allfiles[allfiles.size() - 1];
	}
protected:
    std::vector<CStr> allfiles;
} sTmpList;

/*const char *launchCVS(const char *dir, const CvsArgs & args)
{
	CStr filename;
	FILE *file = launchCVS(dir, args, filename);
	if(file == 0L)
		return 0L;

	fclose(file);
	return sTmpList.push_back(filename);
}*/

#ifdef WIN32
void WaitForCvs(void)
{
	// pump messages manually
	while(1)
	{
		if(sLastWin32CvsThread == 0L)
			return;

		DWORD dw = ::WaitForSingleObject(*sLastWin32CvsThread, 300);

		switch(dw) 
		{ 
			case WAIT_OBJECT_0: 
			case WAIT_TIMEOUT: 
				break;
			case WAIT_ABANDONED: 
			default: 
				cvs_err("An error occured while waiting for cvs to return (error %d)\n", ::GetLastError());
				return;
				break;
		}

		// pump message, but quit on WM_QUIT
		if (!AfxGetThread()->PumpMessage())
		{
			AfxPostQuitMessage(0);
			return;
		}
	}
}
#endif
