/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * Author : Alexandre Parenteau <aubonbeurre@geocities.com> --- February 1998
 */

/*
 * Persistent.cpp --- utilities to store persitent values
 */

#include "stdafx.h"
#include <stdlib.h>
#include <string.h>

#include "Persistent.h"
#include "AppConsole.h"
#include "CPStr.h"

#	ifdef _DEBUG
#	define new DEBUG_NEW
#	undef THIS_FILE
	static char THIS_FILE[] = __FILE__;
#	endif

int CPersistent::fNumEntries = 0;
CPersistent::CPersistentEntry *CPersistent::fAllEntries = 0L;

void CPersistent::Register(CPersistent *value, const char *uniqueName)
{
	for(int i = 0; i < fNumEntries; i++)
	{
		if(strcmp(uniqueName, fAllEntries[i].fUniqueName) == 0)
		{
			ASSERT(0);
		}
	}
	if(fNumEntries == 0)
	{
		//garbyNew(); /*RANGECHECK*/
		fAllEntries = (CPersistentEntry *)malloc(sizeof(CPersistentEntry));
	}
	else
	{
		fAllEntries = (CPersistentEntry *)realloc(fAllEntries,
			(fNumEntries + 1) * sizeof(CPersistentEntry));
	}

	ASSERT(fAllEntries != 0L);
	fAllEntries[fNumEntries].fUniqueName = uniqueName;
	fAllEntries[fNumEntries++].fValue = value;
}

void CPersistent::UnRegister(CPersistent *value)
{
	int i;
	for(i = 0; i < fNumEntries; i++)
	{
		if(value == fAllEntries[i].fValue)
			break;
	}
	ASSERT(i != fNumEntries);
	if(i == fNumEntries)
		return;
	
	memmove(&fAllEntries[i], &fAllEntries[i + 1],
		(fNumEntries - i - 1) * sizeof(CPersistentEntry));
	
	if(--fNumEntries == 0)
	{
		free(fAllEntries);
		fAllEntries = 0L;
		//garbyRemove(0, 0); /*RANGECHECK*/
	}
	else
	{
		fAllEntries = (CPersistentEntry *)realloc(fAllEntries,
			fNumEntries * sizeof(CPersistentEntry));
	}
}


void CPersistent::SaveAll(void)
{
	for(int i = 0; i < fNumEntries; i++)
	{
		CPersistentEntry & entry = fAllEntries[i];
		unsigned int size = entry.fValue->SizeOf();
		if(size == 0)
			continue;
		
		if(!AfxGetApp()->WriteProfileBinary(PROFILE_NAME, entry.fUniqueName,
			(BYTE *)entry.fValue->GetData(), size))
		{
			cvs_err("Error while saving preferences !\n");
			break;
		}
	}
}

bool CPersistent::LoadAll(void)
{
	bool found_one = false;
	for(int i = 0; i < fNumEntries; i++)
	{
		CPersistentEntry & entry = fAllEntries[i];
		BYTE* ppData;
		UINT pBytes;

		if(!AfxGetApp()->GetProfileBinary(PROFILE_NAME, entry.fUniqueName,
			&ppData, &pBytes))
		{
			continue;
		}
		found_one = true;
		entry.fValue->SetData(ppData, pBytes);
		delete ppData;
	}
	return found_one;
}

CPersistent::CPersistentEntry * CPersistent::Find(const char *uniqueName)
{
	for(int i = 0; i < fNumEntries; i++)
	{
		CPersistentEntry & entry = fAllEntries[i];
		if(!strcmp(uniqueName, entry.fUniqueName))
			return &fAllEntries[i];
	}
	return 0L;
}
