#include "stdafx.h"
#include "AraxisMerge.h"
#include "comdef.h"

AraxisMerge::Result AraxisMerge::DoComparison(FilePath file, FilePath diff, FilePath ancestor,
		string diffTitle, string ancestorTitle)
{
  Merge::IApplicationPtr merge;

  // Create a Merge application object. This will cause Merge
  // to be started if it isn't already running.
  HRESULT hr(
    CoCreateInstance(
      __uuidof(Merge::Application), 0, CLSCTX_SERVER,
      __uuidof(Merge::IApplication), (void**)&merge));

  if (SUCCEEDED(hr))
    {
      try
        {
          Merge::IFileComparisonPtr comparison = merge->FileComparison;
		  if (!ancestor.IsEmpty())
		  {
              comparison->put_ViewType(Merge::fvtThreeWay);
			  if (comparison->GetViewType()!=Merge::fvtThreeWay)
			  {
				  return NO_THREE_WAY;
			  }
		  }
          // Make Merge visible and active.
          if (merge->Minimized) merge->Restore();
          merge->Visible = VARIANT_TRUE;

          if (merge->Preferences->Longs->Item[Merge::clAppMax]>0)
			  merge->Maximized = true;

          merge->Active = VARIANT_TRUE;
 
          // Compare the files...

          if (ancestor.IsEmpty())
            {
              // Set file panel titles.
              comparison->SetPanelTitles(file.FilePathString(), 
                  diffTitle.c_str());

              // Compare two files.
              comparison->Compare(file.FilePathString(),
                diff.FilePathString());
            }
          else
            {
              // Set file panel titles.
                comparison->SetPanelTitles(file.FilePathString(), ancestorTitle.c_str(), 
                  diffTitle.c_str());

              // Compare three files.
              comparison->Compare(file.FilePathString(), ancestor.FilePathString(),
                diff.FilePathString());
			  comparison->SaveFileName=file.FilePathString();

            }

          // Hand off control to the user. This means that Merge
          // won't close until the user decides to close it.
          merge->GiveUserControl();
          comparison->GiveUserControl();

        }
      catch (...) 
        {
          hr = E_FAIL;
		  return FAILURE;
        }
	  return SUCCESS;
    }

	return FAILURE;
}
