// CreateModule.cpp : implementation file
//

#include "stdafx.h"
#include "CreateModule.h"
#include "Eskimo.h"
#include "Running.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CreateModule dialog


CreateModule::CreateModule(Eskimo* eskimo_, CWnd* pParent /*=NULL*/)
	: eskimo(eskimo_), CDialog(CreateModule::IDD, pParent), findOnly(false),
	canChangePath(false), canCreateModule(true), findResult(Eskimo::CANCELLED)
{
	//{{AFX_DATA_INIT(CreateModule)
	m_LocalPath = _T("");
	m_Module = _T("");
	m_Status = _T("");
	m_RootStatus = _T("");
	//}}AFX_DATA_INIT
    
//    HMODULE kernel = GetModuleHandle("Kernel");
}

void CreateModule::SetFindMode(bool canChangePath_, bool canCreateModule_)
{
	canChangePath=canChangePath_;
	canCreateModule=canCreateModule_;
	findOnly=true;
}

void CreateModule::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CreateModule)
	DDX_Control(pDX, IDC_CREATE_MODULE, m_CreateModule);
	DDX_Control(pDX, IDC_LOCAL_PATH, m_LocalPathEdit);
	DDX_Control(pDX, IDC_MODULE, m_ModuleDropDown);
	DDX_Control(pDX, IDC_VENDOR, m_VendorDropDown);
	DDX_Control(pDX, IDC_CVSROOT, m_CvsRootDropDown);
	DDX_Control(pDX, IDC_CREATE_REPOSITORY, m_CreateRepository);
	DDX_Text(pDX, IDC_LOCAL_PATH, m_LocalPath);
	DDX_Text(pDX, IDC_STATUS, m_Status);
	DDX_Text(pDX, IDC_ROOT_STATUS, m_RootStatus);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CreateModule, CDialog)
	//{{AFX_MSG_MAP(CreateModule)
	ON_BN_CLICKED(IDC_CREATE_REPOSITORY, OnCreateRepository)
	ON_BN_CLICKED(IDC_CREATE_MODULE, OnCreateModule)
	ON_BN_CLICKED(IDC_CONNECT, OnConnect)
	ON_BN_CLICKED(IDC_BUTTON2, OnCheck)
	ON_BN_CLICKED(IDC_BUTTON3, OnCheckModule)
	ON_CBN_EDITCHANGE(IDC_CVSROOT, OnChangeCvsroot)
	ON_CBN_CLOSEUP(IDC_CVSROOT, OnChangeCvsroot)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/*void CreateModule::Error(string message, string caption)
{
	MessageBox(message.c_str(), caption.c_str(), MB_OK | MB_ICONERROR);
}*/

/////////////////////////////////////////////////////////////////////////////
// CreateModule message handlers

void CreateModule::OnCreateRepository() 
{
	UpdateData();
	// TODO: Add your control notification handler code here
	if (!eskimo->IsRootLocal(string(m_CVSROOT)))
	{
		MessageBox("Igloo can only create a CVS repository\nwith and local path",
			"Can't create repository", MB_OK | MB_ICONERROR);
	}
	else
	{
        BeginWaitCursor();
		string result=eskimo->CreateRepository(string(m_CVSROOT));
		settings.SetRoot(string(m_CVSROOT));
		settings.Save();
        EndWaitCursor();
		MessageBox(result.c_str(), "Repository creation", MB_OK | MB_ICONINFORMATION);
	}
	UpdateStatus();	
}

void CreateModule::OnCreateModule() 
{
	UpdateStatus();	
	if (m_Module.GetLength()==0)
	{
		MessageBox("No module specified", "Module creation", MB_OK | MB_ICONINFORMATION);
		return;
	}
	if (m_LocalPath.GetLength()==0)
	{
		MessageBox("No local path", "Module creation", MB_OK | MB_ICONINFORMATION);
		return;
	}
	EnableWindow(false);
    BeginWaitCursor();

	if (findOnly)
	{
		FilePath localPath=m_LocalPath;
		string result;
		if (!localPath.IsDirectory())
		{
			result=string(localPath.FilePathString()) + " is not a directory";
			MessageBox(result.c_str(), "Module creation", MB_OK | MB_ICONINFORMATION);
			EndWaitCursor();
			EnableWindow(true);
			return;
		}
		vector<string> modules;
		if (!eskimo->TestRoot(string(m_CVSROOT), result, modules))
		{
			MessageBox(result.c_str(), "Module creation", MB_OK | MB_ICONINFORMATION);
			EndWaitCursor();
			EnableWindow(true);
			return;
		}
		if (eskimo->TestModule(string(m_CVSROOT), string(m_Module), result))
		{
			MessageBox("Module already exists in the respository", "Module creation", MB_OK | MB_ICONINFORMATION);
			EndWaitCursor();
			EnableWindow(true);
			return;
		}
		findResult=Eskimo::CREATE_MODULE;
		EndDialog(IDOK);
		EndWaitCursor();
		EnableWindow(true);
		return;
	}


	string result;
	bool success=eskimo->CreateModule(string(m_LocalPath), string(m_CVSROOT),
		string(m_Module), string(m_Vendor), "new", result);
	if (success)
	{
		settings.SetRoot(string(m_CVSROOT));
		settings.SetVendor(string(m_Vendor));
		settings.Save();
 		string result2;
		success=eskimo->AttachModule(string(m_LocalPath), string(m_CVSROOT),
			string(m_Module), result2);
		result = result+"\n"+result2;
	}
    EndWaitCursor();
	EnableWindow(true);
	if (success)
	{
		MessageBox(result.c_str(), "Module creation", MB_OK | MB_ICONINFORMATION);
	}
	else
	{
		MessageBox(result.c_str(), "Module creation", MB_OK | MB_ICONERROR);
	}
	UpdateStatus();	
	if (success) EndDialog(IDOK);
}

void CreateModule::OnOK() 
{
	// TODO: Add extra validation here
	
	CDialog::OnOK();
}

void CreateModule::OnCancel() 
{
	// TODO: Add extra cleanup here
	
	CDialog::OnCancel();
}

void CreateModule::OnConnect() 
{
	UpdateStatus();	
	EnableWindow(false);
    BeginWaitCursor();
	string result;
	bool success = true;
	if (m_Module.GetLength()==0)
	{
		result="No module specified";
		success=false;
	}
	if (success)
	{
		success=eskimo->TestModule(string(m_CVSROOT), string(m_Module), result);
	}
	if (success)
	{
		settings.SetRoot(string(m_CVSROOT));
		settings.Save();
	}
	if (success && !findOnly)
	{
		Running running(eskimo);
		success=eskimo->CheckoutModule(string(m_LocalPath), string(m_CVSROOT),
		string(m_Module), result);
		EndWaitCursor();
		EnableWindow(true);
		if (running.ShouldStop())
		{
			result = string("Operation cancelled");
		}
	}
	else
	{
		EndWaitCursor();
		EnableWindow(true);
	}
	if (success && findOnly && m_LocalPath.GetLength()==0)
	{
		result="No local path specified";
		success=false;
	}
	if (success && findOnly)
	{
		FilePath localPath=m_LocalPath;
		if (!localPath.IsDirectory())
		{
			if (localPath.Exists())
			{
				result=string(localPath.FilePathString()) + " is not a directory";
				success=false;
			}
			else
			{
				FilePath parent=localPath.ParentDirectory();
				if (!parent.IsDirectory())
				{
					result=string(parent.FilePathString()) + " is not a directory";
					success=false;
				}
				else
				{
					if (!CreateDirectory(localPath.FilePathString(), NULL))
					{
						result=string("Could not create ") + string(localPath.FilePathString());
						success=false;
					}
					RemoveDirectory(localPath.FilePathString());
				}
			}
		}
		if (success)
		{
			if (!localPath.IsDirectoryEmpty())
			{
				result=string(localPath.FilePathString()) + " is not empty";
				success=false;
			}
			else
			{
				findResult=Eskimo::FOUND_MODULE;
			}
		}
	}
	if (!findOnly || !success) MessageBox(result.c_str(), "Module connection", MB_OK | MB_ICONINFORMATION);
	UpdateStatus();
	if (success) EndDialog(IDOK);
}

CString CreateModule::GetModule()
{
	return m_Module;
}

static void AddItem(CComboBox& box, CString item)
{
	if (item.GetLength()==0) return;
	int c=box.FindString(-1, item);
	if (c>=0) return;
	box.AddString(item);
}

void CreateModule::AddModule(string module)
{
	AddItem(m_ModuleDropDown, module.c_str());
}

BOOL CreateModule::OnInitDialog() 
{
	BOOL result=CDialog::OnInitDialog();
	m_LocalPathEdit.SetReadOnly(!canChangePath);
	m_CreateModule.EnableWindow(canCreateModule);
	if (!canCreateModule) m_CreateModule.ShowWindow(SW_HIDE);
//	CRichEditCtrl().SetEventMask(pmyRichEditCtrl->GetEventMask() |
//    ENM_CHANGE);
	AddItem(m_ModuleDropDown, m_ModuleGuess);
	AddItem(m_ModuleDropDown, m_ProjectName);
	m_ModuleDropDown.SetWindowText(m_ModuleGuess.GetLength()>0?m_ModuleGuess:m_ProjectName);

	int roots=settings.GetNumberRoots();
	if (roots==0) AddItem(m_CvsRootDropDown, m_CVSROOT);
	for (int i=0; i<roots; i++)
	{
		string root=settings.GetRoot(i);
		AddItem(m_CvsRootDropDown, root.c_str());
		if (i+1==1) AddItem(m_CvsRootDropDown, m_CVSROOT);
	}
	if (roots>0) m_CVSROOT=settings.GetRoot(0).c_str();
	m_CvsRootDropDown.SetWindowText(m_CVSROOT);

    char owner[1000];
    owner[0]=0;
//    LoadString(kernel, 514, owner, 1000);
    char organization[1000];
    organization[0]=0;
//    LoadString(kernel, 515, organization, 1000);
    char username[1000];
    DWORD type;
    DWORD length=1000;
    GetUserName(username, &length);
    HKEY hKey;
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
        "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion",
        0, KEY_QUERY_VALUE, &hKey)!=ERROR_SUCCESS)
    {
        RegOpenKeyEx(HKEY_LOCAL_MACHINE,
        "SOFTWARE\\Microsoft\\Windows\\CurrentVersion",
        0, KEY_QUERY_VALUE, &hKey);
    }
    length=1000;
    RegQueryValueEx(hKey, "RegisteredOwner", NULL, &type, (BYTE*)owner, &length);
    length=1000;
    RegQueryValueEx(hKey, "RegisteredOrganization", NULL, &type, (BYTE*)organization, &length);


	int vendors=settings.GetNumberVendors();
	if (vendors==0)
	{
		m_Vendor="";
		if (strlen(organization)>0)
		{
			AddItem(m_VendorDropDown, organization);
			if (m_Vendor.GetLength()==0) m_Vendor=organization;
		}
		if (strlen(owner)>0 )
		{
			AddItem(m_VendorDropDown, owner);
			if (m_Vendor.GetLength()==0) m_Vendor=owner;
		}
		if (strlen(username)>0)
		{
			AddItem(m_VendorDropDown, username);
			if (m_Vendor.GetLength()==0) m_Vendor=username;
		}
		if (m_Vendor.GetLength()==0)
		{
			AddItem(m_VendorDropDown, "Jalindi");
			m_Vendor="Jalindi";
		}
	}
	else
	{
		for (int i=0; i<vendors; i++)
		{
			string root=settings.GetVendor(i);
			AddItem(m_VendorDropDown, root.c_str());
		}
		m_Vendor=settings.GetVendor(0).c_str();
	}
	m_VendorDropDown.SetWindowText(m_Vendor);


	UpdateStatus();	
	return result;
}


void CreateModule::OnChangeCvsroot() 
{
	UpdateStatus();	
}

void CreateModule::UpdateData(bool update) 
{
	CDialog::UpdateData(update);
	if (update)
	{
		if (m_CvsRootDropDown.GetCurSel()!= CB_ERR )
		{
			m_CvsRootDropDown.GetLBText(m_CvsRootDropDown.GetCurSel(), m_CVSROOT);
		}
		else
		{
			m_CvsRootDropDown.GetWindowText(m_CVSROOT);
		}
		if (m_VendorDropDown.GetCurSel()!= CB_ERR )
		{
			m_VendorDropDown.GetLBText(m_VendorDropDown.GetCurSel(), m_Vendor);
		}
		else
		{
			m_VendorDropDown.GetWindowText(m_Vendor);
		}
		if (m_ModuleDropDown.GetCurSel()!= CB_ERR )
		{
			m_ModuleDropDown.GetLBText(m_ModuleDropDown.GetCurSel(), m_Module);
		}
		else
		{
			m_ModuleDropDown.GetWindowText(m_Module);
		}
	}
}

void CreateModule::UpdateStatus() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.
	
	// TODO: Add your control notification handler code here
	UpdateData();
/*	char buffer[1000];
	m_rootEdit.GetLine(0, buffer, 1000);
	if (buffer[0]==':')*/
	if (!eskimo->IsRootLocal(string(m_CVSROOT)))
	{
		m_CreateRepository.EnableWindow(false);
		m_RootStatus="Cannot determine status of remote repository";
	}
	else
	{
		FilePath cvsroot=eskimo->GetCvsRootPath(string(m_CVSROOT));
		FilePath cvsmain=cvsroot;
		if (cvsroot.IsDirectory()) cvsmain.Append("CVSROOT");
		if (cvsmain.IsDirectory())
		{
			m_RootStatus="Repository already exists in the path specified";
			m_CreateRepository.EnableWindow(false);
		}
		else
		{
			m_RootStatus="Click on create to make a repository in the path specified";
			m_CreateRepository.EnableWindow(true);
		}
	}
	FilePath localPath=m_LocalPath;
	if (localPath.IsControlledDirectory())
	{
		m_Status="Connected to CVS";
	}
	else
	{
		m_Status="Local Path is not connected to CVS. Press create or connect";
	}
	UpdateData(false);	
}

void CreateModule::OnCheck() 
{
	// TODO: Add your control notification handler code here
	EnableWindow(false);
    BeginWaitCursor();
	string result;
	vector<string> modules;
	if (eskimo->TestRoot(string(m_CVSROOT), result, modules))
	{
		settings.SetRoot(string(m_CVSROOT));
		settings.Save();

		m_ModuleDropDown.ResetContent();
		AddItem(m_ModuleDropDown, m_ModuleGuess);
		AddItem(m_ModuleDropDown, m_ProjectName);
		for (vector<string>::iterator it=modules.begin();
		it!=modules.end(); it++)
		{
			AddItem(m_ModuleDropDown, (*it).c_str());
		}
		if (m_ModuleDropDown.GetCount()>0)
		{
			CString top;
			m_ModuleDropDown.GetLBText(0, top);
			m_ModuleDropDown.SetWindowText(top);
		}
	}
    EndWaitCursor();
	EnableWindow(true);
	MessageBox(result.c_str(), "CVS Root Connection Test", MB_OK | MB_ICONINFORMATION);	
}

void CreateModule::OnCheckModule() 
{
	UpdateStatus();	
	if (m_Module.GetLength()==0)
	{
		MessageBox("No module specified", "Module Connection Test", MB_OK | MB_ICONINFORMATION);
		return;
	}
    BeginWaitCursor();
	EnableWindow(false);
	string result;
	eskimo->TestModule(string(m_CVSROOT), string(m_Module), result);
	EnableWindow(true);
    EndWaitCursor();
	MessageBox(result.c_str(), "Module Connection Test", MB_OK | MB_ICONINFORMATION);	
	
}
