// DiffDialog.cpp : implementation file
//

#include "stdafx.h"
#include "igloo.h"
#include "DiffDialog.h"
#include "Eskimo.h"
#include "IglooDialogs.h"
#include "UserSettings.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static const char* none="None";

/////////////////////////////////////////////////////////////////////////////
// DiffDialog dialog

static void AddItem(CComboBox& box, CString item)
{
	if (item.GetLength()==0) return;
	int c=box.FindString(-1, item);
	if (c>=0) return;
	box.AddString(item);
}

DiffDialog::DiffDialog(Eskimo* eskimo_, FilePath filePath_, CWnd* pParent /*=NULL*/)
	: CDialog(DiffDialog::IDD, pParent), eskimo(eskimo_), ok(false)
{
	//{{AFX_DATA_INIT(DiffDialog)
	m_FilePath = _T("");
	m_Revision = -1;
	m_Exe = _T("");
	m_Type = -1;
	m_Ancestor = -1;
	//}}AFX_DATA_INIT
	m_FilePath=filePath_.FilePathString();
	UserSettings settings;
	diffType=settings.GetDiffType();
	m_Type=(int)diffType;
	if (m_Type==UserSettings::REGULAR)
		m_Exe=settings.GetDiffExe().c_str();
	for (int i=0; i<4; i++)
	{
		revisions=eskimo->Revisions(filePath_);
		if (revisions.size()>0) break;
		Sleep(100);
	}
}

BOOL DiffDialog::OnInitDialog()
{
	BOOL result=CDialog::OnInitDialog();
	SetTypeSensitivity();
	int count=0;
	string ancestorRevision=none;
	AddItem(m_Ancestors, ancestorRevision.c_str());
	RevisionVector::iterator iterator=revisions.begin();
	for (;count<30 && iterator!=revisions.end(); iterator++, count++)
	{
		Revision revision= *iterator;
		AddItem(m_Revisions, revision.GetLabel().c_str());
		AddItem(m_Ancestors, revision.GetLabel().c_str());
		if (revision.IsSoleAncestor()) ancestorRevision=revision.GetLabel();
	}
	if (revisions.size()>0)
	{
		m_Revisions.SelectString(0, revisions[0].GetLabel().c_str());
	}
	m_Ancestors.SelectString(0, ancestorRevision.c_str());
	if (eskimo->Dialogs()->ShouldStop()) return false;
//	m_Revisions.SetWindowText(head);
	return result;
}


void DiffDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(DiffDialog)
	DDX_Control(pDX, IDC_EXE_LABEL, m_ExeLabel);
	DDX_Control(pDX, IDC_ANCESTOR_LABEL, m_AncestorLabel);
	DDX_Control(pDX, IDC_ANCESTOR, m_Ancestors);
	DDX_Control(pDX, IDC_BROWSE, m_ExeBrowse);
	DDX_Control(pDX, IDC_EXE, m_ExeBox);
	DDX_Control(pDX, IDC_REVISION, m_Revisions);
	DDX_Text(pDX, IDC_FILEPATH, m_FilePath);
	DDX_CBIndex(pDX, IDC_REVISION, m_Revision);
	DDX_Text(pDX, IDC_EXE, m_Exe);
	DDX_CBIndex(pDX, IDC_TYPE, m_Type);
	DDX_CBIndex(pDX, IDC_ANCESTOR, m_Ancestor);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(DiffDialog, CDialog)
	//{{AFX_MSG_MAP(DiffDialog)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	ON_CBN_EDITCHANGE(IDC_TYPE, OnEditchangeType)
	ON_CBN_SELCHANGE(IDC_TYPE, OnSelchangeType)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// DiffDialog message handlers

void DiffDialog::OnOK() 
{
	UpdateData(true);
	ok=true;
	revisionIndex=m_Revisions.GetCurSel();
	ancestorIndex=m_Ancestors.GetCurSel()-1;
	UserSettings settings;
	if (diffType==UserSettings::REGULAR)
	{
		settings.SetDiffExe(string(m_Exe));
	}
	settings.SetDiffType(diffType);
	settings.Save();
	CDialog::OnOK();
}

bool DiffDialog::WasCancelled()
{
	return !ok;
}

Revision DiffDialog::GetRevision()
{
	return revisions[revisionIndex];
}

Revision DiffDialog::GetAncestor()
{
	if (ancestorIndex<0) return Revision();
	return revisions[ancestorIndex];
}


void DiffDialog::OnBrowse() 
{
	UpdateData(true);
	static char BASED_CODE szFilter[] = "Executable Files (*.exe)|*.exe|All Files (*.*)|*.*||";
	const char* exe=0;
	if (m_Exe.GetLength()>0)
	{
		exe=(const char*)m_Exe;
	}
	// TODO: Add your control notification handler code here
	CFileDialog browse( TRUE, "*.Exe", exe, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		szFilter, this );
	browse.DoModal();
	m_Exe=browse.GetPathName();
	if (diffType==UserSettings::REGULAR)
	{
		UserSettings settings;
		settings.SetDiffExe(string(m_Exe));
		settings.Save();
	}
	UpdateData(false);
}

void DiffDialog::OnEditchangeType() 
{
	// TODO: Add your control notification handler code here
}

void DiffDialog::SetTypeSensitivity()
{
	m_ExeBox.EnableWindow(diffType==UserSettings::REGULAR);
	m_ExeBrowse.EnableWindow(diffType==UserSettings::REGULAR);
	m_ExeLabel.EnableWindow(diffType==UserSettings::REGULAR);
	m_Ancestors.EnableWindow(diffType==UserSettings::ARAXIS_PRO);
	m_AncestorLabel.EnableWindow(diffType==UserSettings::ARAXIS_PRO);
	if (diffType!=UserSettings::ARAXIS_PRO)
	{
		m_Ancestors.SelectString(0, none);
	}
}

string DiffDialog::GetExe()
{
	return m_Exe;
}

UserSettings::DiffType DiffDialog::GetDiffType()
{
	return diffType;
}

void DiffDialog::OnSelchangeType() 
{
	UpdateData(true);
	UserSettings settings;
	if (diffType==UserSettings::REGULAR)
	{
		settings.SetDiffExe( string(m_Exe));
	}
	m_Exe="";
	if (m_Type==0)
	{
		diffType=UserSettings::CVS;
	}
	else if(m_Type==1)
	{
		diffType=UserSettings::REGULAR;
		m_Exe=settings.GetDiffExe().c_str();
	}
	else if(m_Type==2)
	{
		diffType=UserSettings::ARAXIS;
	}
	else if(m_Type==3)
	{
		diffType=UserSettings::ARAXIS_PRO;
	}
	settings.SetDiffType(diffType);
	settings.Save();
	SetTypeSensitivity();
	UpdateData(false);
}
