#include "stdafx.h"
#include "Entries.h"

#include <fstream>
#include <locale>
#include "CriticalSection.h"

class EntryType
{
public:
    enum Type { FILENAME=0, VERSION=1, DATE=2} ;
};

typedef map<FilePath, Entries*> EntriesCache;
EntriesCache entriesCache;
CriticalSection lock;

Entries& Entries::GetEntries(FilePath entriesFile)
{
	lock.Enter();
	EntriesCache::iterator iterator=entriesCache.find(entriesFile);
	Entries* entries=0;
	if (iterator!=entriesCache.end())
	{
		entries=(*iterator).second;
		// If the file has changed then we recreate the entry
		if (entries->timeStamp!=entriesFile.GetTimeStamp())
		{
			delete entries;
			entriesCache.erase(iterator);
			entries=0;
		}
	}
	if (entries==0)
	{
		entries=new Entries(entriesFile);
		entriesCache.insert(EntriesCache::value_type(entriesFile, entries));
	}
	lock.Leave();
	return *entries;
}

Entries::ClearCache()
{
	lock.Enter();
	for (EntriesCache::const_iterator iterator=entriesCache.begin();
	iterator!=entriesCache.end(); iterator++)
	{
		delete (*iterator).second;
	}
	entriesCache.erase(entriesCache.begin(), entriesCache.end());
	lock.Leave();
}

string getEntryItem(string entryLine, EntryType::Type entryType)
{
    int index=0;
    for (int i=0; i<entryLine.length(); i++)
    {
        if (entryLine[i]=='/')
        {
            if (index==entryType)
            {
                i++;
                int start=i;
                for (; i<entryLine.length(); i++)
                {
                    if (entryLine[i]=='/')
                    {
                        entryLine = entryLine.substr(0, i);
                        break;
                    }
                }
                return entryLine.substr(start);
            }
            else
            {
                index++;
            }
        }
    }
    return "";
}

static bool matchFilename(char *line, string file)
{
	string entriesFilename=getEntryItem(line, EntryType::FILENAME);
	return _stricmp(entriesFilename.c_str(), file.c_str())==0;
}

static string toLowerCase(string s)
{
	char *lower=new char[s.length()+1];
	for (int i=0; i<s.length(); i++)
	{
		lower[i]=tolower(s[i]);
	}
	lower[i]=0;
	s=lower;
	delete [] lower;
	return s;
}

Entries::Entries(FilePath entries_) : entries(entries_.FilePathString()),
timeStamp(entries_.GetTimeStamp())
{
    ifstream inFile(entries.c_str());
    if (!inFile)
    {
        return;
    }
    char s[10000];
    while (inFile.getline(s,10000))
    {
        string filename=getEntryItem(s, EntryType::FILENAME);
		filename = toLowerCase(filename);
		entriesMap[filename]=string(s);
    }
    inFile.close();
}

string Entries::GetEntry(string file)
{
	lock.Enter();
	EntriesMap::const_iterator iterator=entriesMap.find(toLowerCase(file));
	string result = (iterator==entriesMap.end())?"":(*iterator).second;
	lock.Leave();
	return result;
}


string Entries::GetEntryTimeStamp(string file)
{
	string entry=GetEntry(file);
	if (entry.length()==0) return "";
	return getEntryItem(entry, EntryType::DATE);
}

string Entries::GetVersion(string file)
{
	string entry=GetEntry(file);
	if (entry.length()==0) return "";
	return getEntryItem(entry, EntryType::VERSION);
}

bool Entries::IsFileControlled(string file)
{
	string entry=GetEntry(file);
	return (entry.length()!=0);
}

FilePathVector Entries::GetEntriesFiles(string directory) const
{
	FilePathVector files;
	lock.Enter();
	for (EntriesMap::const_iterator iterator=entriesMap.begin(); iterator!=entriesMap.end();
	iterator++)
	{
		string entry=(*iterator).second;
		files.push_back(directory+"\\"+getEntryItem(entry, EntryType::FILENAME));
	}
	lock.Leave();
	return files;
}
