#include "stdafx.h"
#include <process.h>
#include <windows.h>

#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "GoCVS.h"
#include <CvsArgs.h>
#include <fstream>
#include <strstream>

#include "Eskimo.h"
#include "IglooDialogs.h"
#include "FileList.h"
#include "UserSettings.h"
#include "FilesMonitor.h"
#include "Running.h"

class EskimoBody
{
public:
    EskimoBody(string aUser, ostream& stream, FilesMonitor* filesMonitor_) : user(aUser), iout(stream),
		dialogs(), errorLogged(true), filesMonitor(filesMonitor_)
	{
	}
    ~EskimoBody() 
	{
	}
    int Cvs(CvsArgs & args, const FilePathVector&  files);
    int Cvs(FilePath path, const CvsArgs & args);
    int CvsCapture(CvsArgs & args, const FilePathVector&  files, string& result, bool winCommand=true);
    int CvsCapture(FilePath path, const CvsArgs & args, string& result, bool winCommand=true);
    int CvsNull(FilePath path, const CvsArgs & args);
    string GetCommentArg();
	void DeleteCommentFile();
	void ClearNewCommand();
	void Version();

    ostream& iout;
    string projectName;
    FilePath projectPath;
    string user;
    string comment;
    IglooDialogs dialogs;
	FilesMonitor* filesMonitor;
	GoCvs cvs;
    UserSettings settings;
	bool newCommand;
	bool errorLogged;
	string commentFile;
};

void Eskimo::ErrorLogged()
{
	body->errorLogged=true;
}

bool Eskimo::GetLatest(const FilePathVector&  files)
{
    CvsArgs args;
    args.add("update");
    int exit=body->Cvs(args, files);
    return exit==0;
}

Eskimo::FileChanged Eskimo::NextFileChanged()
{
	FileChanged changed;
	return changed;
}

int Eskimo::FilesChanged()
{
	return 0;
}

bool Eskimo::Update(const FilePath& directory, bool recursive)
{
    CvsArgs args;
    args.add("update");
	if (!recursive)
	{
		args.add("-l");
	}
    int exit=body->Cvs(directory, args);
    return exit==0;
}


bool Eskimo::IsRootLocal(string cvsroot)
{
	if (cvsroot.find(":")==string.npos) return true;
	if (cvsroot.length()<=1) return false;
	if (cvsroot[0]!=':' && cvsroot[1]==':' && cvsroot.substr(2).find(":")==string.npos) return true;
	if (cvsroot.substr(0, strlen(":local:"))==":local:") return true;
	return false;
}

int EskimoBody::Cvs(CvsArgs& args, const FilePathVector&  files)
{
	if (files.size()==0) return -1;
	for (int i=0; i<files.size(); ++i) args.add(files[i].FileName());
	return Cvs(files[0].DirectoryName(),  args);
}

int EskimoBody::Cvs(FilePath path, const CvsArgs & args)
{
    iout << path << ">" ;
    for (int i=0; i<args.Argc(); i++)
    {
        iout << " " << args.Argv()[i];
    }
    iout << endl;
	FilePath currentDir=FilePath::GetCurrentDirectoryFilePath();
    int exit=cvs.Launch(path, args, &iout);
	currentDir.SetCurrentDirectory();
	return exit;
}

int EskimoBody::CvsCapture(CvsArgs& args, const FilePathVector&  files, string& result, bool withCommand)
{
	if (files.size()==0) return -1;
	for (int i=0; i<files.size(); ++i) args.add(files[i].FileName());
	return CvsCapture(files[0].DirectoryName(),  args, result, withCommand);
}

int EskimoBody::CvsCapture(FilePath path, const CvsArgs & args, string& result, bool withCommand)
{
    ostrstream stringiout;
	if (withCommand)
	{
		stringiout << path << ">" ;
		for (int i=0; i<args.Argc(); i++)
		{
			stringiout << " " << args.Argv()[i];
		}
		stringiout << endl;
	}
	FilePath currentDir=FilePath::GetCurrentDirectoryFilePath();
    int exit= cvs.Launch(path, args, &stringiout);
    result = string(stringiout.str(), stringiout.pcount());
	currentDir.SetCurrentDirectory();
    return exit;
}

int EskimoBody::CvsNull(FilePath path, const CvsArgs & args)
{
	FilePath currentDir=FilePath::GetCurrentDirectoryFilePath();
    int exit=cvs.Launch(path, args);
	currentDir.SetCurrentDirectory();
	return exit;
}

FilePath Eskimo::GetDiffRevision(const FilePath& filePath, string revision, bool ancestor)
{
	FilePath diffDir=filePath.DirectoryName();
	diffDir.Append("CVS");
	if (!diffDir.Exists()) return FilePath("invalid");
	diffDir.Append(ancestor?"Ancestor":"Diff");
	if (!diffDir.Exists()) CreateDirectory(diffDir.FilePathString(), NULL);
	
	CvsArgs args;
	args.add("update");
	args.add("-p");
	args.add("-r");
	args.add(revision);
	args.add(filePath.FileName());

	string text;
	int exit=body->CvsCapture(filePath.DirectoryName(),  args, text);
	if (exit!=0)
	{
		iout << text;
		return FilePath("invalid");
	}

	FilePath diffPath=diffDir;
	diffPath.Append(filePath.FileName());
	if (diffPath.Exists())
	{
		diffPath.SetWriteable();
		::DeleteFile(diffPath.FilePathString());
	}
	ofstream output(diffPath.FilePathString());

	istrstream input(text.c_str());
	char l[1000];
	bool started=false;
	const char* start="***********";
	while (input.getline(l, 1000))
	{
		if (started) output << l << endl;
		if (strncmp(l, start, strlen(start))==0) started=true;
	}
	output.close();
	diffPath.SetReadOnly();
	return diffPath;	
}

bool Eskimo::Checkout(const FilePathVector&  files)
{
    // If working offline then don't check them out. Just set them to 
    // writeable.
    if (body->settings.WorkOffline())
    {
        for (int i=0; i< files.size(); i++)
        {
            files[i].SetWriteable();
        }
        return true;
    }
    {
        CvsArgs args;
        args.add("editors");
		if (body->settings.WarnOnEdit())
		{
			string result;
			int exit=body->CvsCapture(args, files, result, false);
			if (result.length()>0)
			{
				if (!body->dialogs.Question(
					string("The following files are already being edited\n")+
					result+"Are you sure you want to continue?",
					"Files are already being edited"))
				{
					return false;
				}
			}
		}
		else
		{
			int exit=body->Cvs(args, files);
		}
    }
    // If we are working independently or working offline then
    // don't get the latest version of this stuff.
    if (body->settings.AutoCommit()) GetLatest(files);
    CvsArgs args;
    args.add("edit");
    int exit=body->Cvs(args, files);
    return exit==0;
}

void Eskimo::ProcessSomeMessages()
{
    MSG msg;
    for (int i=0; i<1000 && ::PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE); ++i)
    {
//        GetMessage(&msg, (HWND) NULL, 0, 0)
            TranslateMessage(&msg); 
        DispatchMessage(&msg); 
    }
}

bool Eskimo::UnCheckout(const FilePathVector&  files)
{
    CvsArgs args;
    args.add("unedit");
    int exit=body->Cvs(args, files);
    return exit==0;
}

string EskimoBody::GetCommentArg()
{
/*	if (comment.find("\n")!=string::npos)
	{
		commentFile = _tempnam( getenv("TEMP"), "cvscomment" );
		ofstream out(commentFile.c_str());
		out << comment;
		out.close();
		string commentarg = string("-F") + commentFile;
		return commentarg;
	}
	else*/
	{
		string c;
		for (int i=0; i<comment.length(); i++)
		{
			if (comment[i]!=13) c+=comment[i];
		}
		string commentarg = string("-m") + (c.length()==0?"no message":c);// + "";
		return commentarg;
	}
}

void EskimoBody::DeleteCommentFile()
{
	if (commentFile.length()>0)
	{
		DeleteFile(commentFile.c_str());
		commentFile="";
	}
}

bool Eskimo::Checkin(const FilePathVector& files)
{
    CvsArgs args;
    args.add("commit");
    args.add(body->GetCommentArg());
    int exit=body->Cvs(args, files);
	FilePathVector checkedOutFiles;
	for (int i=0; i<files.size(); ++i)
	{
		if (files[i].GetBackupFilePath().Exists())
		{
			checkedOutFiles.push_back(files[i]);
		}
	}
	if (exit==0 && checkedOutFiles.size()>0)
	{
		CvsArgs args2=CvsArgs();
		args2.add("unedit");
		body->dialogs.SetAutoNo(true);
		exit=body->Cvs(args2, files);
		body->dialogs.SetAutoNo(false);
	}
	for (i=0; i<files.size(); ++i)
	{
		if (files[i].IsTimeStampSameAsEntry())
		{
			files[i].SetReadOnly();
		}
	}
    return exit==0;
}

bool Eskimo::CheckinKeep(const FilePathVector& files)
{
    CvsArgs args;
    args.add("commit");
    args.add(body->GetCommentArg());
    int exit=body->Cvs(args, files);
	FilePathVector checkedInFiles;
	for (int i=0; i<files.size(); ++i)
	{
		if (!(files[i].GetBackupFilePath().Exists()))
		{
			checkedInFiles.push_back(files[i]);
		}
	}
	if (checkedInFiles.size()>0) return Checkout(checkedInFiles);
    return exit==0;
}

void Eskimo::SetComment(const char*  aComment)
{
    if (aComment) body->comment=string(aComment);
    else body->comment=string();
}


bool Eskimo::IsDifferent(FilePath filePath)
{
    if (!filePath.Exists()) return true;
	// If this is the zero version then the file has not been commited yet
	if (filePath.IsZeroVersion()) return false;
	// If we are working offline then this is ?probably? being called to check if there
	// is something to update. Because we are offline then we are not going to
	// update it so I'll just report that there are no differences.
    if (body->settings.WorkOffline() || body->settings.WorkIndependently())
	{
		return false;
	}
    CvsArgs args;
    args.add("diff");
    args.add("--brief");
    args.add(filePath.FileName());
    int exit=body->CvsNull(filePath.DirectoryName(),  args);
    if (exit == 1) return true;
    return false;
}

bool Eskimo::ShowDifference(FilePath filePath)//, string& difference)
{
    if (!filePath.Exists())
	{
		iout << "Local file does not exist : " << filePath;
		return true;
	}
	// If this is the zero version then the file has not been commited yet
	if (filePath.IsZeroVersion())
	{
		iout << "The file has not been committed to the repository : " << filePath;
		return false;
	}

	string difference;
	bool cvsDifference= body->dialogs.Diff(filePath, difference);
	iout << difference;
	return cvsDifference;
	

    // This returns a null string just now!
/*    CvsArgs args;
    args.add("diff");
    args.add(filePath.FileName());
    int exit=body->CvsCapture(filePath.DirectoryName(),  args, difference);
    if (exit == 1) return true;
    return false;*/
}

bool Eskimo::IsDifferent(FilePath filePath, string revision, string& difference)
{
    if (!filePath.Exists())
	{
		iout << "Local file does not exist : " << filePath;
		return true;
	}
	// If this is the zero version then the file has not been commited yet
	if (filePath.IsZeroVersion())
	{
		iout << "The file has not been committed to the repository : " << filePath;
		return false;
	}

    CvsArgs args;
    args.add("diff");
	if (revision!="head")
	{
		args.add("-r");
		args.add(revision);
	}
    args.add(filePath.FileName());
    int exit=body->CvsCapture(filePath.DirectoryName(),  args, difference);
    if (exit == 1) return true;
    return false;
}

/**
	All the files are from the same directory
*/
string Eskimo::History(const FilePathVector&  files)
{
	if (files.size()==0) return "";
    CvsArgs args;
    args.add("log");
    string history;
    int exit=body->CvsCapture(args, files,  history);
    return history;
}

RevisionVector Eskimo::Revisions(const FilePath& path)
{
	FilePathVector files;
	files.push_back(path);
	string history=History(files);
	istrstream input(history.c_str());
	RevisionVector revisions;
	static string revisionLabel="revision ";
	static string headLabel="head: ";
	static string dateLabel="date: ";
	static string authorLabel="author: ";
	static string marker=";";
	string head;
	string ancestor=path.GetBaseRevision();
	char l[1000];
	while (input.getline(l, 1000))
	{
		string line=l;
		if (line.find(headLabel)==0)
		{
			head=line.substr(headLabel.size());
		}
		if (line.find(revisionLabel)==0)
		{
			string revision=line.substr(revisionLabel.size());
			input.getline(l, 1000);
			line=l;
			Revision rev(revision, revision==head, revision==ancestor);
			size_t datePos=line.find(dateLabel);
			if (datePos!=string::npos)
			{
				datePos+=dateLabel.length();
				rev.SetDate(line.substr(datePos,line.find(marker, datePos)-datePos));
			}
			size_t authorPos=line.find(authorLabel);
			if (authorPos!=string::npos)
			{
				authorPos+=authorLabel.length();
				rev.SetAuthor(line.substr(authorPos,line.find(marker, authorPos)-authorPos));
			}
//			iout << "'" << revision << "'" << endl;
			revisions.push_back(rev);
		}
	}
	return revisions;
}


bool Eskimo::Watchers(const FilePathVector& files)
{
    CvsArgs args;
    args.add("watchers");
    int exit=body->Cvs(args, files);
    return exit==0;
}

bool Eskimo::Editors(const FilePathVector& files)
{
    CvsArgs args;
    args.add("editors");
    int exit=body->Cvs(args, files);
    return exit==0;
}

bool Eskimo::WatchAdd(const FilePathVector& files)
{
    CvsArgs args;
    args.add("watch");
    args.add("add");
    int exit=body->Cvs(args, files);
    return exit==0;
}

bool Eskimo::WatchRemove(const FilePathVector& files)
{
    CvsArgs args;
    args.add("watch");
    args.add("remove");
    int exit=body->Cvs(args, files);
    return exit==0;
}

bool Eskimo::Commit(const FilePathVector& files)
{
    CvsArgs args;
    args.add("commit");
//    args.add(body->GetCommentArg());
    int exit=body->Cvs(args, files);
	if (exit==0)
	{
		for (int i=0; i<files.size(); i++)
		{
			files[i].SetReadOnly();
		}
	}
    return exit==0;
}


bool Eskimo::GoIgloo(const FileList& fileLists)
{
	int count=0;
    for (int j=0; j<fileLists.NumberOfLists(); j++)
    {
        const FilePathVector& files=fileLists.GetList(j);
        for (int i=0; i<files.size(); i++)
        {
            iout << files[i] << endl;
			count++;
        }
    }
    ExtraCommands command=Dialogs()->Settings(count);
	if (command==NoCommand) return true;
	if (command==LoginCommand)
	{
		if (body->projectPath.IsValid())
		{
			Login(body->projectPath);
		}
		else
		{
			if (getenv("CVSROOT")==NULL)
			{
				LoginRoot("");
			}
			else
			{
				LoginRoot(getenv("CVSROOT"));
			}
		}
		return true;
	}
	if (command==LogoutCommand)
	{
		if (body->projectPath.IsValid())
		{
			Logout(body->projectPath);
		}
		else
		{
			if (getenv("CVSROOT")==NULL)
			{
				LogoutRoot("");
			}
			else
			{
				LogoutRoot(getenv("CVSROOT"));
			}
		}
		return true;
	}
    for (j=0; j<fileLists.NumberOfLists(); j++)
    {
        const FilePathVector& files=fileLists.GetList(j);
		switch (command)
		{
		case WatchersCommand:
			Watchers(files);
			break;
		case EditorsCommand:
			Editors(files);
			break;
		case WatchAddCommand:
			WatchAdd(files);
			break;
		case WatchRemoveCommand:
			WatchRemove(files);
			break;
		case CommitCommand:
			Commit(files);
			break;
		case ConnectCommand:
			body->dialogs.CreateModuleAndConnection(body->projectName, 
				files[0].ParentDirectory(),
				files[0].GetRepository());
			break;
		}
	}
    return true;
}

string Eskimo::GetProjectName()
{
	return body->projectName;
}

bool Eskimo::GoIgloo(HWND parentWindow, ostream& stream)
{
    IglooDialogs dialogs;
    dialogs.SetParentWindow(parentWindow);
    ExtraCommands command=dialogs.Settings(0);
	if (command==NoCommand) return true;
	Eskimo eskimo("", stream, NULL);
	eskimo.SetParentWindow(parentWindow);
	switch (command)
	{
	case LoginCommand:
		{
			if (getenv("CVSROOT")==NULL)
			{
				eskimo.LoginRoot("");
			}
			else
			{
				eskimo.LoginRoot(getenv("CVSROOT"));
			}
		}
		break;
	case LogoutCommand:
		{
			if (getenv("CVSROOT")==NULL)
			{
				eskimo.LogoutRoot("");
			}
			else
			{
				eskimo.LogoutRoot(getenv("CVSROOT"));
			}
		}
		break;
	}
    return true;
}

FilePathVector Eskimo::GetFileList()
{
	CvsArgs args;
//    args.add("-n");
    args.add("update");
    int exit=body->Cvs(body->projectPath, args);
	FilePathVector files=body->projectPath.GetEntriesFiles();
	return files;
}

void EskimoBody::Version()
{
	static int firstTime=1;
    if (firstTime)
    {
        if (getenv("CVSROOT")==NULL)
        {
            iout << "Jalindi Igloo version 0.988, CVSROOT is not set\n";
        }
        else
        {
            iout << "Jalindi Igloo version 0.988, CVSROOT = " << getenv("CVSROOT") << endl;
        }
        CvsArgs args;
        args.add("--version");
		FilePath dir=FilePath::GetCurrentDirectoryFilePath();
        Cvs(dir, args);
        firstTime=0;
    }
}


Eskimo::FindModuleResult Eskimo::FindModule(string &module, FilePath& localPath,
		string& cvsRoot, bool canChangePath, bool canCreateModule)
{
//	body->Version();
	iout << "Find Module\n";
	if (localPath.IsValid() && !localPath.IsEmpty() && localPath.IsControlledDirectory())
//	FilePath workspace(localPath.GetWorkspace());
//	if (workspace.IsValid())
	{
		return FOUND_MODULE;
	}
	if (localPath.IsEmpty())
	{
		char dir[500];
		GetCurrentDirectory(500, dir);	
		localPath=dir;
		canChangePath=true;
	}
    return body->dialogs.FindCreateModuleAndConnection(module, localPath, cvsRoot,
		canChangePath, canCreateModule);
}


bool Eskimo::OpenModule(string module, FilePath localPath, string cvsRoot, bool silent)
{
    body->projectName=module;
    body->projectPath=localPath.GetWorkspace();
	body->Version();
	if (strlen(localPath.FilePathString())==0)
	{
        iout << "Unknown project:" << body->projectName << " - no local path " << endl;
		iout << "Accepting project anyway\n";
/*        string message=string("Unknown project: ") + projectName + "\n";
        message+=  "There was no local path specified. Accepting project anyway\n";
        message+=  "Current Directory: " + string(dir) + "\n";
        MessageBox(parentWindow, message.c_str(), "Unknown Project", MB_OK | MB_ICONWARNING);*/
        return true;
	}
    if (!localPath.IsControlledDirectory())
    {
		boolean success=true;
		if (silent)
		{
			Running running(this);
			string result;
			success=CheckoutModule(localPath, cvsRoot,module, result);
/*			if (running.ShouldStop())
			{
				result = string("Operation cancelled");
			}
			if (!success)
			{
				body->dialogs.MessageBox(result, "Module connection", MB_OK | MB_ICONINFORMATION);
			}*/
		}
		if (!silent || !success)
		{
			iout << "Uncontrolled project:" << body->projectName << " - local " << localPath << " - project path: " << body->projectPath <<  endl;
			/*        string message=string("Uncontrolled project: ") + projectName + "\n";
			message=message + "Local path: " + localPath.filePathString() + "\n";
			message=message + "Project path: " + projectPath.filePathString() + "\n";
			dialogs.Error(message, "Uncontrolled Project");*/
			body->dialogs.CreateModuleAndConnection(body->projectName, localPath, localPath.GetRepository());
		}
        if (localPath.IsControlledDirectory())
        {
            iout << "project is now controlled " << endl;
        }
        else
        {
            iout << "project is still not controlled " << endl;
        }
        return localPath.IsControlledDirectory();
    }
    iout << "Opened CVS project: " << body->projectName << " - local " << localPath << " - project path " << body->projectPath <<  endl;
    iout << "Project root: " << localPath.GetProjectRoot() << endl;
//  iout << "project is under CVS " << endl;
    return true;
}

bool Eskimo::CreateModule(string module, FilePath localPath, string cvsRoot)
{
    body->projectName=module;
    body->projectPath=localPath.GetWorkspace();
	body->Version();
	if (strlen(localPath.FilePathString())==0)
	{
        iout << "Unknown project:" << body->projectName << " - no local path " << endl;
		iout << "Accepting project anyway\n";
        return true;
	}
    if (!localPath.IsControlledDirectory())
    {
		string result;
		if (TestModule(cvsRoot, module, result))
		{
			return OpenModule(module, localPath, cvsRoot, true);
		}
		boolean success=true;
		{
			Running running(this);
			UserSettings settings;
			string vendor=settings.GetVendor(0);
			success=CreateModule(localPath, cvsRoot, module, vendor, "new", result);
			if (success)
			{
				settings.SetRoot(string(cvsRoot));
				settings.Save();
				string result2;
				success=AttachModule(localPath, cvsRoot, module, result2);
				result = result+"\n"+result2;
			}
		}
		if (!success)
		{
			body->dialogs.MessageBox(result, "Module connection", MB_OK | MB_ICONINFORMATION);
		}
		if (!success)
		{
			iout << "Uncontrolled project:" << body->projectName << " - local " << localPath << " - project path: " << body->projectPath <<  endl;
			body->dialogs.CreateModuleAndConnection(body->projectName, localPath, localPath.GetRepository());
		}
        if (localPath.IsControlledDirectory())
        {
            iout << "project is now controlled " << endl;
        }
        else
        {
            iout << "project is still not controlled " << endl;
        }
        return localPath.IsControlledDirectory();
    }
    iout << "Opened CVS project: " << body->projectName << " - local " << localPath << " - project path " << body->projectPath <<  endl;
    iout << "Project root: " << localPath.GetProjectRoot() << endl;
//  iout << "project is under CVS " << endl;
    return true;
}

FilePath Eskimo::GetCvsRootPath(string cvsrootstring)
{
	if (cvsrootstring.substr(0, strlen(":local:"))==":local:") cvsrootstring = cvsrootstring.substr(strlen(":local:"));
	int length=cvsrootstring.length();
	if (length==3 && cvsrootstring[1]==':') return cvsrootstring;
	if (length>0 && (cvsrootstring[length-1]=='\\' || cvsrootstring[length-1]=='/'))
	{
		return cvsrootstring.substr(0, length-1);
	}
	return cvsrootstring;
}

string Eskimo::GetCvsRootArg(string cvsrootstring)
{
	if (!Eskimo::IsRootLocal(cvsrootstring))
	{
		int length=cvsrootstring.length();
		if (length>0 && (cvsrootstring[length-1]=='\\' || cvsrootstring[length-1]=='/'))
		{
			return cvsrootstring.substr(0, length-1);
		}
		return cvsrootstring;
	}
	string cvsrootpath=GetCvsRootPath(cvsrootstring).FilePathString();
//	if (cvsrootstring.substr(0, strlen(":local:"))==":local:") return cvsrootstring;
	return string(":local:")+cvsrootpath;
}

bool Eskimo::CreateModule(FilePath localPath, string cvsrootstring, string module,
						  string vendor, string release, string& result)
{
	// Make the vendor string valid
	for (int i=0; i<vendor.size(); i++)
	{
		if (vendor[i]==' ') vendor[i]='_';
	}
	FilePath cvsroot=GetCvsRootPath(cvsrootstring);
    CvsArgs args;
    args.add("-d");
    args.add(GetCvsRootArg(cvsrootstring));
    args.add("import");
    args.add("-m Jalindi");
    args.add("-I *");
    args.add(module);
    args.add(vendor);
    args.add(release);
    int exit=body->CvsCapture(localPath, args, result);
	int ignore = result.find("\nI");
	while (ignore>=0)
	{
		int endignore = result.find("\n", ignore+1);
		result.erase(ignore, endignore-ignore);
		ignore = result.find("\nI");
	}
	iout << result << endl;
	return exit==0;
}

bool Eskimo::TestRoot(string cvsroot, string &result, vector<string> &modules)
{
	FilePath testDir=FilePath::GetTempDirectoryFilePath();
	CvsArgs args;
    args.add("-d");
    args.add(GetCvsRootArg(cvsroot));
    args.add("checkout");
    args.add("-s");
    int exit=body->CvsCapture(testDir, args, result);
	iout << result << endl;
	if (result.find("cvs login")!=result.npos)
	{
		if (!LoginRoot(cvsroot))
		{
			result = string("Unsuccessful login to CVSROOT: ")+cvsroot;
		}
        else
        {
		    exit=body->CvsCapture(testDir, args, result);
			iout << result << endl;
        }
	}
	if (exit!=0)
	{
		result = "Cannot verify the existence of "+GetCvsRootArg(cvsroot)+"\n"+result;
		return false;
	}
	istrstream input(result.c_str());
	char l[1000];
	while (input.getline(l, 1000))
	{
		string line=l;
		if (line.size()>0 && line[0]!='#')
		{
			size_t cvsrootpos=line.find("CVSROOT");
			if (cvsrootpos==line.npos)
			{
				string module;
				size_t space=line.find(" ");
				if (space!=line.npos) module=line.substr(0, space);
				iout << "Found module: " << module << endl;
				modules.push_back(module);
			}
		}
	}
	result = "CVSROOT : Tested Ok";

	return true;
    
	
/*	FilePath testDir=FilePath::GetTempDirectoryFilePath();
    testDir.Append("JalindiTestDir");
    testDir.CreateDirectory();
    FilePath fakeModule=testDir;
    fakeModule.Append("CVSROOT");
    FilePath moduleFilePath=fakeModule;
    moduleFilePath.Append("modules");
	CvsArgs args;
    args.add("-d");
    args.add(GetCvsRootArg(cvsroot));
    args.add("export");
    args.add("-n");
    args.add("-D 1/1/2038");
    args.add("CVSROOT/modules");
    int exit=body->CvsCapture(testDir, args, result);
	iout << result << endl;
	if (result.find("cvs login")!=result.npos)
	{
		if (!LoginRoot(cvsroot))
		{
			result = string("Unsuccessful login to CVSROOT: ")+cvsroot;
		}
        else
        {
		    exit=body->CvsCapture(testDir, args, result);
			iout << result << endl;
        }
	}
	bool success=false;
	if (result.find("U CVSROOT/modules")!=result.npos)
	{
		result = "CVSROOT : Tested Ok";
		success=true;
		ifstream moduleFile(moduleFilePath.FilePathString());
		char l[1000];
		while (moduleFile.getline(l, 1000))
		{
			string line=l;
			if (line.size()>0 && line[0]!='#')
			{
				size_t cvsrootpos=line.find("CVSROOT");
				if (cvsrootpos==line.npos)
				{
					string module;
					size_t space=line.find(" ");
					if (space!=line.npos) line=line.substr(0, space);
					space=line.find(9);
					if (space!=line.npos)
					{
						module=line.substr(0, space);
						line=line.substr(space+1);
						space=line.find(9);
						if (space!=line.npos) line=line.substr(0, space);
					}
					iout << "Found module: " << module << " : " << line << endl;
					if (line.length()>0) modules.push_back(line);
				}
			}
		}
		moduleFile.close();
	}
//	if (exit==0) result = string("Successfully connected to module: ") + module + string(", using: ") + cvsroot.filePathString();
    DeleteFile(moduleFilePath.FilePathString());
    RemoveDirectory(fakeModule.FilePathString());
    RemoveDirectory(testDir.FilePathString());
	return success;*/
}

bool Eskimo::TestModule(string cvsroot, string module, string &result)
{
//	return true;
    FilePath testDir=FilePath::GetTempDirectoryFilePath();
    testDir.Append("JalindiTestDir");
    testDir.CreateDirectory();
    FilePath fakeModule=testDir;
    fakeModule.Append(module);
	CvsArgs args;
    args.add("-n");
    args.add("-d");
    args.add(GetCvsRootArg(cvsroot));
    args.add("checkout");
    args.add(module);
    int exit=body->CvsCapture(testDir, args, result);
	iout << result << endl;
	if (result.find("cvs login")!=result.npos)
	{
		if (!LoginRoot(cvsroot))
		{
			result = string("Unsuccessful login to CVSROOT: ")+cvsroot;
		}
        else
        {
		    exit=body->CvsCapture(testDir, args, result);
		    iout << result << endl;
        }
	}
	bool success=false;
	if (result.find("in directory ")!=result.npos)
	{
		result = module + " : Tested Ok";
		success=true;
	}
	// Sometimes the checkout failes but we still know the module is there
	if (result.find("end of file from server")!=result.npos)
	{
		result = module + " : Tested Ok!";
		success=true;
	}
//	if (exit==0) result = string("Successfully connected to module: ") + module + string(", using: ") + cvsroot.filePathString();
    RemoveDirectory(fakeModule.FilePathString());
    RemoveDirectory(testDir.FilePathString());
	return success;
}

bool Eskimo::LoginRoot(string cvsroot)
{
    FilePath testDir=FilePath::GetTempDirectoryFilePath();
    testDir.Append("JalindiTestDir");
    testDir.CreateDirectory();
	IglooDialogs::PasswordRootPair loginPair=body->dialogs.Login(cvsroot);
	CvsArgs args;
    args.add("-d");
    args.add(GetCvsRootArg(loginPair.second));
    args.add("login");
	string result;
	body->cvs.SetPassword(loginPair.first);
    int exit=body->CvsCapture(testDir, args, result);
	iout << result << endl;
	if (exit==0)
	{
		body->dialogs.Info(result, "CVS Login attempt");
	}
	else
	{
		body->dialogs.Error(result, "CVS Login attempt");
	}
    RemoveDirectory(testDir.FilePathString());
	return exit==0;
}

bool Eskimo::Login(FilePath localPath)
{
	string password=body->dialogs.Login(localPath.GetProjectRoot()).first;
	if (password.length()==0) return false;
	CvsArgs args;
    args.add("login");
	string result;
	body->cvs.SetPassword(password);
    int exit=body->CvsCapture(localPath, args, result);
	iout << result << endl;
	if (exit==0)
	{
		body->dialogs.Info(result, "CVS Login attempt");
	}
	else
	{
		body->dialogs.Error(result, "CVS Login attempt");
	}
	return exit==0;
}

bool Eskimo::LogoutRoot(string cvsroot)
{
    FilePath testDir=FilePath::GetTempDirectoryFilePath();
    testDir.Append("JalindiTestDir");
    testDir.CreateDirectory();
	CvsArgs args;
    args.add("-d");
    args.add(GetCvsRootArg(cvsroot));
    args.add("logout");
	string result;
    int exit=body->CvsCapture(testDir, args, result);
	iout << result << endl;
	body->dialogs.Error(result, "CVS Logout attempt");
    RemoveDirectory(testDir.FilePathString());
	return exit==0;
}

bool Eskimo::Logout(FilePath localPath)
{
	CvsArgs args;
    args.add("logout");
	string result;
    int exit=body->CvsCapture(localPath, args, result);
	iout << result << endl;
	body->dialogs.Error(result, "CVS Logout attempt");
	return exit==0;
}

IglooDialogs* Eskimo::Dialogs()
{
	return &body->dialogs;
}

bool moreLines(string result)
{
	return result.length()>0;
}

string nextLine(string& result)
{
	int index=result.find('\n');
	if (index>=0)
	{
		string line=result.substr(0, index);
		result=result.substr(index+1);
		return line;
	}
	string line=result;
	result="";
	return line;
}


bool Eskimo::CheckoutModule(FilePath localPath, string cvsrootstring, string module, string &result)
{
    if (localPath.IsControlledDirectory())
    {
        result = string(localPath.FilePathString()) + " is already controlled by CVS\n";
		return true;
    }
	if (localPath.Exists())
	{
		if (!localPath.IsDirectoryEmpty())
		{
			result = string(localPath.FilePathString()) + " is not empty. Cannot checkout module\n";
			return false;
		}
		RemoveDirectory(localPath.FilePathString());
	}
	FilePath parent=localPath.ParentDirectory();
	if (!parent.IsDirectory())
	{
			result = string(parent.FilePathString()) + " is not a directory. Cannot checkout module\n";
			return false;
	}
	FilePath cvsroot=GetCvsRootPath(cvsrootstring);
	CvsArgs args;
    args.add("-d");
    args.add(GetCvsRootArg(cvsrootstring));
    args.add("checkout");
    args.add("-d");
    args.add(localPath.FileName().c_str());
    args.add(module);
    int exit=body->Cvs(parent, args);
/*    FilePath cvsDir=localPath;
    cvsDir.Append("CVS");
    if (!cvsDir.CreateDirectory())
	{
		result = string("Failed to create : ")+cvsDir.FilePathString();
		return false;
	}
    FilePath rootPath=cvsDir;
    rootPath.Append("Root");
    ofstream rootStream(rootPath.FilePathString());
    if (!rootStream)
	{
		result = string("Failed to connect");
		return false;
	}

    rootStream << GetCvsRootArg(cvsrootstring) << endl;
    rootStream.close();

    FilePath repositoryPath=cvsDir;
    repositoryPath.Append("Repository");
    ofstream repositoryStream(repositoryPath.FilePathString());
    if (!repositoryStream)
	{
		result = string("Failed to connect");
		return false;
	}
    repositoryStream << module << endl;
    repositoryStream.close();

    FilePath entriesPath=cvsDir;
    entriesPath.Append("Entries");
    ofstream entriesStream(entriesPath.FilePathString());
    if (!entriesStream)
	{
		result= string("Failed to connect");
		return false;
	}
    entriesStream.close();

	FilePath cvsroot=GetCvsRootPath(cvsrootstring);
	CvsArgs args;
    args.add("-d");
    args.add(GetCvsRootArg(cvsrootstring));
    args.add("update");
    args.add("-d");
    int exit=body->Cvs(localPath, args);*/
	if (exit==0) result = string("Successfully connected to module: ") + module + string(", using: ") + cvsroot.FilePathString();
    else
    {
/*        DeleteFile(rootPath.FilePathString());
        DeleteFile(entriesPath.FilePathString());
        DeleteFile(repositoryPath.FilePathString());
        RemoveDirectory(cvsDir.FilePathString());*/
		result = string("Failed to connected to module: ") + module + string(", using: ") + cvsroot.FilePathString() + string("\n See log for details");
    }
	return exit==0;
}

bool Eskimo::AttachModule(FilePath localPath, string cvsrootstring, string module, string &result)
{
    if (localPath.IsControlledDirectory())
    {
        result = string(localPath.FilePathString()) + " is already controlled by CVS\n";
		return true;
    }
	if (!localPath.Exists())
	{
		result = string(localPath.FilePathString()) + " does not exist. Cannot attach module\n";
		return false;
	}
	FilePath parent=localPath.ParentDirectory();
	if (!parent.IsDirectory())
	{
			result = string(parent.FilePathString()) + " is not a directory. Cannot checkout module\n";
			return false;
	}
    FilePath cvsDir=localPath;
    cvsDir.Append("CVS");
    if (!cvsDir.CreateDirectory())
	{
		result = string("Failed to create : ")+cvsDir.FilePathString();
		return false;
	}
    FilePath rootPath=cvsDir;
    rootPath.Append("Root");
    ofstream rootStream(rootPath.FilePathString());
    if (!rootStream)
	{
		result = string("Failed to connect");
		return false;
	}

    rootStream << GetCvsRootArg(cvsrootstring) << endl;
    rootStream.close();

    FilePath repositoryPath=cvsDir;
    repositoryPath.Append("Repository");
    ofstream repositoryStream(repositoryPath.FilePathString());
    if (!repositoryStream)
	{
		result = string("Failed to connect");
		return false;
	}
    repositoryStream << module << endl;
    repositoryStream.close();

    FilePath entriesPath=cvsDir;
    entriesPath.Append("Entries");
    ofstream entriesStream(entriesPath.FilePathString());
    if (!entriesStream)
	{
		result= string("Failed to connect");
		return false;
	}
    entriesStream.close();

	FilePath cvsroot=GetCvsRootPath(cvsrootstring);
	CvsArgs args;
    args.add("-d");
    args.add(GetCvsRootArg(cvsrootstring));
    args.add("update");
    args.add("-d");
    int exit=body->Cvs(localPath, args);
	if (exit==0) result = string("Successfully attached to module: ") + module + string(", using: ") + cvsroot.FilePathString();
    else
    {
        DeleteFile(rootPath.FilePathString());
        DeleteFile(entriesPath.FilePathString());
        DeleteFile(repositoryPath.FilePathString());
        RemoveDirectory(cvsDir.FilePathString());
		result = string("Failed to attached to module: ") + module + string(", using: ") + cvsroot.FilePathString() + string("\n See log for details");
    }
	return exit==0;
}


string Eskimo::CreateRepository(string cvsrootstring)
{
	FilePath cvsroot=GetCvsRootPath(cvsrootstring);
	if (!cvsroot.Exists())
	{
		if (!cvsroot.CreateDirectory()) return "failed to create CVSROOT directory";
	}
	if (!cvsroot.IsDirectory())
	{
		return "CVSROOT is not a directory";
	}
	FilePath cvsmain=cvsroot;
	cvsmain.Append("CVSROOT");
	if (cvsmain.IsDirectory())
	{
		return "repository already exists";
	}

    CvsArgs args;
    args.add("-d");
    args.add(GetCvsRootArg(cvsrootstring));
	args.add("init");
	string result;
    int exit=body->CvsCapture(cvsroot.ParentDirectory(), args, result);
	iout << result << endl;
	return result;
}

/*string Eskimo::GetProjectPath(string project)
{
    return string();
}*/

bool Eskimo::RemoveFile(FilePath filePath)
{
    CvsArgs args;
    args.add("remove");
    args.add("-f");
    args.add(filePath.FileName());
    int exit=body->Cvs(filePath.DirectoryName(),  args);
    if (exit==0 && body->settings.AutoCommit())
    {
        CvsArgs args2;
        args2.add("commit");
        args2.add(body->GetCommentArg());
        args2.add(filePath.FileName());
        exit=body->Cvs(filePath.DirectoryName(),  args2);
    }
    return exit==0;
}

bool Eskimo::AddFile(FilePath filePath)
{
    FilePath localPath=filePath.ParentDirectory();
    if (!localPath.IsControlledDirectory())
    {
        FilePath workspace=filePath.GetWorkspace();
        string module=filePath.GetRepository();
 /*       if (workspace.IsValid())
        {
            int length=strlen(workspace.ParentDirectory().FilePathString());
            module=string(localPath.FilePathString()).substr(length);
            for (int i=0; i<module.length(); ++i)
            {
                if (module[i]=='\\') module[i]='/';
            }
            if (module.length()>0 && module[0]=='/')
            {
                module=module.substr(1);
            }
        }*/
        if (module.length()==0) module=localPath.FileName();
        body->dialogs.CreateModuleAndConnection(module, localPath, filePath.GetRepository());
        if (localPath.IsControlledDirectory())
        {
            iout << "module is now controlled " << endl;
        }
        else
        {
            iout << localPath << " is not under CVS " << endl;
            return true;
        }
    }
    CvsArgs args;
    args.add("add");
    args.add(body->GetCommentArg());
    args.add(filePath.FileName());
    int exit=body->Cvs(localPath,  args);
    if (exit==0 && body->settings.AutoCommit())
	{
		FilePathVector files;
		files.push_back(filePath);
		return Checkin(files);
	}
    return exit==0;
}

Eskimo::Eskimo(string user, ostream& stream, FilesMonitor* filesMonitor) : body(new EskimoBody(user, stream, filesMonitor)),
iout(stream)
{
	body->dialogs.SetEskimo(this);
	body->ClearNewCommand();
	body->cvs.SetEskimo(this);
}

Eskimo::~Eskimo() 
{
    delete body;
}

void EskimoBody::ClearNewCommand()
{
	newCommand=false;
}

void Eskimo::SetParentWindow(HWND window)
{
    body->dialogs.SetParentWindow(window);
	body->newCommand=true;
	body->errorLogged=false;
    body->settings.Load();
}

bool Eskimo::HasErrorBeenLogged()
{
	return body->errorLogged;
}

Eskimo::FileStatus Eskimo::DoQuery(FilePath filePath)
{
	FileStatus status=NOT_CONTROLLED;
	if (filePath.IsControlled())
	{
		status = CHECKED_IN;
		string timestamp = filePath.GetEntryTimeStamp();
		string fileTimeStamp = filePath.GetTimeStamp();
		// Comparing strings doesn't seem to work!
		if (filePath.Exists() && !filePath.IsTimeStampSameAsEntry())
		{
			status = CHECKED_OUT;
		}
		if (filePath.GetBackupFilePath().Exists())
		{
			status = CHECKED_OUT;
		}
		//            iout << filePath << ":" << timestamp << ":\n" << fileTimeStamp << ":\n" << endl;
		//            iout << filePath << ":" << (status == CHECKED_OUT?"out":"in") << endl;
		//            iout << filePath << ":" << (timestamp==fileTimeStamp?"todate":"nottodate") << endl;
		//            iout << filePath.GetBackupFilePath() << ":" << (filePath.GetBackupFilePath().Exists()?"exists":"not exists") << endl;
	}	
	return status;
}

Eskimo::FileStatus Eskimo::Query(FilePath filePath, bool addToMonitor)
{
	FileStatus status=DoQuery(filePath);
	if (addToMonitor && body->filesMonitor!=0) body->filesMonitor->Monitor(filePath, status);
	return status;
}


Eskimo::FileStatusVector Eskimo::Query(FilePathVector filePaths)
{
    FileStatusVector fileStatus;
    for (int i = 0 ; i< filePaths.size(); i++)
    {
        FilePath filePath=filePaths[i];
		FileStatus status=Query(filePath);
        fileStatus.push_back(status);
    }
    return fileStatus;
}


bool Eskimo::Properties(FilePath filePath)
{
	FilePathVector filePaths;
	filePaths.push_back(filePath);
	FileStatusVector statuses=Query(filePaths);
	FileStatus status=statuses[0];
    if (status==NOT_CONTROLLED)
    {
        iout << filePath << " is not controlled\n";
		return false;
    }
	bool differs = IsDifferent(filePath);
    if (status==CHECKED_IN)
    {
        if (differs)
        {
			iout << filePath << " is not checked out. You don't have the latest version\n";
		}
        else
        {
            iout << filePath << " is not checked out\n";
        }
    }
    else if (differs)
    {
		if (filePath.Exists())
		{
			if (filePath.IsWriteable())
			{
				iout << "You have " << filePath
					<< " checked out and it has changed\n";
			}
			else 
			{
				iout << "You have " << filePath
					<< " checked out somewhere else, you don't have the latest version\n";
			}
		}
		else
		{
			iout << "Either file " << filePath << " does not exist or has an error\n";
		}
    }
    else
    {
		iout << "You have " << filePath << " checked out and have not changed it\n";
    }
	if (!body->settings.WorkOffline())
	{
		Watchers(filePaths);
		Editors(filePaths);
	}
	return true;
}

ostream& Eskimo::GetLogStream()
{
	return iout;
}


