#ifndef _Eskimo_h
#define _Eskimo_h

class CvsArgs;
#include "FilePath.h"
#include "Revision.h"
class EskimoBody;
class IglooDialogs;
class FileList;
class FilesMonitor;
//typedef vector<string> StringVector;

/**
    This is the main class that is called from the IDE via the sccglue
    Do not change the signature of this class. I.e. don't add any
    virtual functions or member variables.
*/
class Eskimo
{
public:
    // These status are checkout, in as far as Visual Studio is concerned
    enum FileStatus {NOT_CONTROLLED, CHECKED_IN, CHECKED_OUT};
	enum FindModuleResult {FOUND_MODULE, CREATE_MODULE, CANCELLED};
	enum ExtraCommands { NoCommand, WatchersCommand, EditorsCommand,
		WatchAddCommand, WatchRemoveCommand,
		CommitCommand, LoginCommand, LogoutCommand, ConnectCommand };
    typedef vector<FileStatus> FileStatusVector;
	struct FileChanged
	{
		FilePath file;
		FileStatus status;
	};

    Eskimo(string aUser, ostream& stream, FilesMonitor* filesMonitor);
    ~Eskimo();

    FindModuleResult FindModule(string &module, FilePath& localPath,
		string& cvsRoot, bool canChangePath, bool canCreateModule);
    bool OpenModule(string module, FilePath localPath, string cvsRoot, bool silent);
    bool CreateModule(string module, FilePath localPath, string cvsRoot);
//    string GetProjectPath(string project);

    bool GetLatest(const FilePathVector& files);
    bool RemoveFile(FilePath filePath);
    bool AddFile(FilePath filePath);
	bool Properties(FilePath filePath);
    bool Watchers(const FilePathVector& files);
    bool Editors(const FilePathVector& files);
    bool WatchAdd(const FilePathVector& files);
    bool WatchRemove(const FilePathVector& files);
    bool Update(const FilePath& directory, bool recursive);
    bool Commit(const FilePathVector& files);
    bool Checkout(const FilePathVector& files);
    bool UnCheckout(const FilePathVector& files);
    bool Checkin(const FilePathVector& files);
    bool CheckinKeep(const FilePathVector& files);
	FilePath GetDiffRevision(const FilePath& file, string revision, bool ancestor=false);
    void SetComment(const char* comment);
    bool IsDifferent(FilePath filePath);
    bool ShowDifference(FilePath filePath);//, string& difference);
    bool IsDifferent(FilePath filePath, string revision, string& difference);
    bool GoIgloo(const FileList& fileLists);
	FilePathVector GetFileList();
    static bool GoIgloo(HWND parentWindow, ostream& stream);
    string History(const FilePathVector& files);
	RevisionVector Revisions(const FilePath& path);
    FileStatusVector Query(FilePathVector filePaths);
	static FileStatus DoQuery(FilePath filePath);
	FileStatus Query(FilePath filePath, bool addToMonitor=true);
	FileChanged NextFileChanged();
	int FilesChanged();
//    int GetFileList(string* files, int& count);
    void SetParentWindow(HWND window);
	string CreateRepository(string cvsroot);
	bool CreateModule(FilePath localPath, string cvsroot,
		string module, string vendor, string release, string& result);
	bool CheckoutModule(FilePath localPath, string cvsroot,
		string module, string &result);
	bool AttachModule(FilePath localPath, string cvsroot,
		string module, string &result);
	bool TestModule(string cvsroot, string module, string &result);
	bool TestRoot(string cvsroot, string &result, vector<string> &modules);
	bool LoginRoot(string cvsroot);
	bool Login(FilePath localPath);
	bool LogoutRoot(string cvsroot);
	bool Logout(FilePath localPath);
	void ErrorLogged();
	bool HasErrorBeenLogged();
    IglooDialogs* Dialogs();
	string GetProjectName();
	
    void ProcessSomeMessages();
	static bool IsRootLocal(string cvsroot);
	static FilePath GetCvsRootPath(string cvsrootstring);
	static string GetCvsRootArg(string cvsrootstring);
	ostream& GetLogStream();

private:
    /*
        This is the implmentation of the eskimo
        do not add anything as the sccglue relies on this class having this
        signature
        */
    EskimoBody* body;
    ostream& iout;
};

#endif // Eskimo