#include "stdafx.h"
#include "FilePath.h"
#include <sys/types.h> 
#include <sys/stat.h> 
#include <time.h>
#include "Entries.h"
#include <fstream>
#include "UserSettings.h"

FilePath FilePath::GetCurrentDirectoryFilePath()
{
    char dir[5000];
    GetCurrentDirectory(5000, dir);
    FilePath path=FilePath(dir);
    return path;
}

FilePath FilePath::GetTempDirectoryFilePath()
{
    char dir[5000];
    GetTempPath(5000, dir);
    FilePath path=FilePath(dir);
    return path;
}

bool FilePath::IsTimeStampSameAsEntry() const
{
	string timestamp = GetEntryTimeStamp();
	string fileTimeStamp = GetTimeStamp();
	// Comparing strings doesn't seem to work!
	// If the filestamps are the same then it looks like we checked it
	// in properly
	return timestamp==fileTimeStamp;
}

bool FilePath::SetCurrentDirectory() const
{
    if (!IsDirectory()) return false;
    ::SetCurrentDirectory(filePath.c_str());
    return true;
}

FilePath::FilePath(string filePath_)
{
    SetFilePath(filePath_);
}

FilePath::FilePath(const char* filePath_)
{
		SetFilePath(filePath_?string(filePath_):string());
}

FilePath::FilePath()
{
}

static string toLowerCase(string s)
{
	char *lower=new char[s.length()+1];
	for (int i=0; i<s.length(); i++)
	{
		lower[i]=tolower(s[i]);
	}
	lower[i]=0;
	s=lower;
	delete [] lower;
	return s;
}


static string toUpperCase(string s)
{
	char *lower=new char[s.length()+1];
	for (int i=0; i<s.length(); i++)
	{
		lower[i]=toupper(s[i]);
	}
	lower[i]=0;
	s=lower;
	delete [] lower;
	return s;
}

bool FilePath::operator< (const FilePath& otherPath) const
{
	return toLowerCase(filePath) < toLowerCase(otherPath.filePath);
//	return filePath<otherPath.filePath;
}

bool FilePath::operator== (const FilePath& otherPath) const
{
	return toLowerCase(filePath).compare(toLowerCase(otherPath.filePath))==0;
//	return filePath.compare(otherPath.filePath)==0;
}

const char* FilePath::FilePathString() const
{
    return filePath.c_str();
}


ostream& operator<<(ostream& str, const FilePath& filePath)
{
	str << filePath.filePath;
	return str;
}

bool FilePath::IsDirectory() const
{
	// Check and see if this is an incomplete remote network path
	// If the name starts with \\ and done not have another separator
	// Then this can't possibly be a directory
	if (filePath.length()>=2 && IsDirSeparator(0) && IsDirSeparator(1))
	{
		bool complete=false;
		for (int i=2; i<filePath.length(); ++i)
		{
			if (IsDirSeparator(i))
			{
				complete=true;
				break;
			}
		}
		if (!complete) return false;
	}
    int result=GetFileAttributes(filePath.c_str());
    if (result<0) return false;
    return (result & FILE_ATTRIBUTE_DIRECTORY) >0;
}

bool FilePath::IsControlledDirectory() const
{
    return EntriesFilePath().Exists();
}

bool FilePath::Exists() const
{
    int result=GetFileAttributes(filePath.c_str());
    if (result<0) return false;
    return true;
}

static bool realFile(string file)
{
	if (file==".") return false;
	if (file=="..") return false;
	return true;
}

bool FilePath::IsDirectoryEmpty() const
{
	if (!IsDirectory()) false;
    FilePath dir=FilePath::GetCurrentDirectoryFilePath();
    WIN32_FIND_DATA findData;
	string path=filePath;
	int length=filePath.length();
	if (length>0 && IsDirSeparator(length-1))
	{
		path+="*";
	}
	else
	{
		path+="\\*";
	}
    HANDLE handle =
    FindFirstFile (path.c_str(), &findData);
	int count=0;
    if (handle != INVALID_HANDLE_VALUE)
    {
		if (realFile(findData.cFileName)) count++;
        while (count==0 && FindNextFile(handle, &findData))
		{
			if (realFile(findData.cFileName)) count++;
		}
    }
    FindClose(handle);
	return count==0;
}

static FilePath GetWorkspaceDir(int level, FilePath child)
{
    if (level>=6) return FilePath("invalid");
    level++;
    FilePath dir=FilePath::GetCurrentDirectoryFilePath();
    WIN32_FIND_DATA findData;
    HANDLE handle =
    FindFirstFile ("CVS", &findData);
    if (handle == INVALID_HANDLE_VALUE)
    {
        dir=child;
    }
    else
    {
        FindClose(handle);
        SetCurrentDirectory("..");
        dir = GetWorkspaceDir(level, child);
    }
    return dir;
}

bool FilePath::IsEmpty() const
{
	return filePath.length()==0;
}

FilePath FilePath::GetWorkspace() const
{
	FilePath dir=*this;
	if (dir.IsEmpty()) return "invalid";
	// Find a directory that is controlled
	bool found=dir.IsControlledDirectory();
	while (!found)
	{
		dir=dir.ParentDirectory();
		if (dir.IsEmpty()) return "invalid";
		found=dir.IsControlledDirectory();
	}
	// Find the leftmost directory that is controlled
	FilePath cvsdir;
	while (found)
	{
		cvsdir=dir;
		dir=dir.ParentDirectory();
		if (dir.IsEmpty()) return cvsdir;
		found=dir.IsControlledDirectory();
	}
	return cvsdir;
/*    FilePath path= GetCurrentDirectoryFilePath();
    FilePath workspace("invalid");
    if (IsDirectory())
    {
        SetCurrentDirectory();
        workspace=GetWorkspaceDir(0, *this);
    }
    path.SetCurrentDirectory();
    return workspace;*/
}

string FilePath::GetTimeStamp() const
{
    struct stat fileStat;
    if (stat (filePath.c_str(), &fileStat) == 0)
    {
        struct tm* time=gmtime(&fileStat.st_mtime);
        if (time)
        {
            string timeString(asctime(time),24);
			return timeString;
        }
    }
    return "";
}

bool FilePath::IsWriteable() const
{
    DWORD result=GetFileAttributes(filePath.c_str());
    if (result<0) return false;
    return (result & FILE_ATTRIBUTE_READONLY) ==0;
}

bool FilePath::SetWriteable() const
{
    DWORD result=GetFileAttributes(filePath.c_str());
    if (result<0) return false;
    DWORD newFileAttributes = result & 
        (FILE_ATTRIBUTE_ARCHIVE | FILE_ATTRIBUTE_COMPRESSED |
        FILE_ATTRIBUTE_DIRECTORY | FILE_ATTRIBUTE_HIDDEN |
        FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_NORMAL |
        FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_TEMPORARY);
    SetFileAttributes(filePath.c_str(), newFileAttributes);
    return true;
}

bool FilePath::SetReadOnly() const
{
    DWORD result=GetFileAttributes(filePath.c_str());
    if (result<0) return false;
    DWORD newFileAttributes = result | FILE_ATTRIBUTE_READONLY;
    SetFileAttributes(filePath.c_str(), newFileAttributes);
    return true;
}

FilePath FilePath::EntriesFilePath() const
{
	if (IsDirectory()) return FilePath(filePath+"\\CVS\\Entries");
    return FilePath(DirectoryName()+"\\CVS\\Entries");
}

Entries& FilePath::Entries() const
{
	return Entries::GetEntries(EntriesFilePath());
}

bool FilePath::IsControlled() const
{
    string fileName=FileName();
    return Entries().IsFileControlled(fileName);
}

string FilePath::GetEntryTimeStamp() const
{
    string fileName=FileName();
    return Entries().GetEntryTimeStamp(fileName);
}

string FilePath::GetBaseRevision() const
{
    string fileName=FileName();
    return Entries().GetVersion(fileName);
}

FilePathVector FilePath::GetEntriesFiles() const
{
	return Entries().GetEntriesFiles(filePath);
}

bool FilePath::IsZeroVersion() const
{
    string fileName=FileName();
    return Entries().GetVersion(fileName)=="0";
}

FilePath FilePath::GetBackupFilePath() const
{
    return FilePath(DirectoryName()+"\\CVS\\Base\\"+FileName());
}

void FilePath::Append(string pathItem)
{
	int length=filePath.length();
	if (length>0 && IsDirSeparator(length-1))
	{
		SetFilePath(filePath+pathItem);
	}
	else
	{
		SetFilePath(filePath+"\\"+pathItem);
	}
}

FilePath FilePath::Mirror(string module)
{
	UserSettings settings;
	FilePath mirror=settings.MirrorPath();
	if (filePath.size()>3 && filePath[1]==':')
	{
		if (IsDirSeparator(2))
		{
			mirror.Append(filePath.substr(3));
		}
		else
		{
			mirror.Append(filePath.substr(2));
		}
	}
	else
	{
		if (IsDirSeparator(0))
		{
			mirror.Append(filePath.substr(1));
		}
		else
		{
			mirror.Append(filePath);
		}
	}
	return mirror;
}


static string GetRealDirName(string fake)
{
    if (fake.length()<=2) return fake;
    WIN32_FIND_DATA findData;
    HANDLE handle =
    FindFirstFile (fake.c_str(), &findData);
    string realName;
    if (handle != INVALID_HANDLE_VALUE)
    {
        realName = findData.cFileName;
        FindClose(handle);
        int endptr=fake.length()-1;
        while ((fake[endptr]!='/' && fake[endptr]!='\\') && endptr>0)
        {
            endptr --;
        }
        fake = fake.substr(0, endptr);
        if (endptr<2)
        {
            return fake+string("\\")+realName;
        }
        return GetRealDirName(fake)+string("\\")+realName;
    }
    return fake;
}

static string GetRealFileName(string fake)
{
    WIN32_FIND_DATA findData;
    HANDLE handle =
    FindFirstFile (fake.c_str(), &findData);
    string realName;
    if (handle != INVALID_HANDLE_VALUE)
    {
        realName = findData.cFileName;
        FindClose(handle);
        int endptr=fake.length()-1;
        while ((fake[endptr]!='/' && fake[endptr]!='\\') && endptr>0)
        {
            endptr --;
        }
        fake = fake.substr(0, endptr);
        if (endptr<2)
        {
            return fake+string("\\")+realName;
        }
        return GetRealDirName(fake)+"\\"+realName;
    }
    else
    {
        int endptr=fake.length()-1;
        while ((fake[endptr]!='/' && fake[endptr]!='\\') && endptr>0)
        {
            endptr --;
        }
        if (endptr > 0)
        {
            string file = fake.substr(endptr+1);
            fake = fake.substr(0, endptr);
            return GetRealDirName(fake)+"\\"+file;
        }
    }
    return fake;
}

bool IsCharDirSeparator(char c)
{
    return (c=='/' || c=='\\');
}

bool FilePath::IsDirSeparator(int i) const
{
    return IsCharDirSeparator(filePath[i]);
}


string FilePath::FileName() const
{
    for (int i=filePath.length()-1; i>=0; i--)
    {
        if (IsDirSeparator(i)) return filePath.substr(i+1);
    }
    return filePath;
}

bool FilePath::CreateDirectory() const
{
    if (!Exists())
	{
		SECURITY_ATTRIBUTES security;
		security.nLength=sizeof(SECURITY_ATTRIBUTES);
		security.lpSecurityDescriptor=NULL;
		security.bInheritHandle=FALSE;
        if (!::CreateDirectory(FilePathString(), &security)) return false;
	}
    return true;
}

FilePath FilePath::ParentDirectory() const
{
	return DirectoryName();
}

string FilePath::DirectoryName() const
{
	if (filePath.length()==3 && filePath[1]==':' && IsDirSeparator(2)) return "";
    for (int i=filePath.length()-1; i>=0; i--)
    {
        if (IsDirSeparator(i))
        {
            return filePath.substr(0, i);
        }
    }
    return "";
}


void FilePath::SetFilePath(string path)
{
	if (path.length()==0)
	{
		filePath=path;
		return;
	}
	if (path.length()==2 && path[1]==':')
	{
		filePath=path+"\\";
		return;
	}
	if (path.length()==3 && path[1]==':')
	{
		filePath=path;
		return;
	}
	if (IsCharDirSeparator(path[path.length()-1])) path=path.substr(0,path.length()-1);
    int result=GetFileAttributes(path.c_str());
    if (result<0)
    {
        filePath=::GetRealFileName(path);
        return;
    }
    if ((result & FILE_ATTRIBUTE_DIRECTORY) >0)
    {
        filePath=::GetRealDirName(path);
    }
    else
    {
        filePath=::GetRealFileName(path);
    }
}

string FilePath::GetProjectRoot() const
{
	FilePath projectPath=*this;
    projectPath.Append("CVS\\Root");
    ifstream inFile(projectPath.FilePathString());
    if (!inFile)
    {
        return "";
    }
    char s[10000];
    inFile.getline(s,10000);
    inFile.close();
    return s;
}

bool FilePath::IsValid() const
{
	return !(filePath=="invalid");
}

// Try to find the module by searching upward on the disk for a CVS directory
// containing a Repository file.
// If found this repository is used appended with our path up to this level.
string FilePath::GetRepository(ForceCase forceCase) const
{
	if (DirectoryName().empty()) return "";
	string subrepository;
	string path;
	FilePath directory=IsDirectory()?filePath:DirectoryName();
	while(1)
	{
		FilePath reposPath = directory;
		reposPath.Append("CVS\\Repository");
		ifstream reposFile(reposPath.FilePathString());
		if(!(!reposFile))
		{
			char s[10000];
			if( reposFile.getline(s,10000) )
			{
				// We have now found a CVS/Repositories file.
				// Append our path to this.

				string repository;
				if(!subrepository.empty())
				{
					if(forceCase==lower)
					{
						subrepository=toLowerCase(subrepository);
					} else if(forceCase==upper)
					{
						subrepository=toUpperCase(subrepository);
					}
					repository = string(s) + "/" + subrepository;
				} else 
					repository = string(s);

				return repository;
			}
			reposFile.close();
		}

		string name=directory.FileName();
		directory=directory.ParentDirectory();
		if (strlen(directory.FilePathString())==0) return "";//subrepository;
		if(subrepository.empty())
		{
			subrepository += name;
		} else
		{
			subrepository = name + "/" + subrepository ;
		}
	}
}