#ifndef _FilePath_h
#define _FilePath_h


#include <string>
#include <ostream>
#include <vector>
#include <set>
using namespace std;

class Entries;

class FilePath
{
public:
    FilePath();
    FilePath(string filePath_);
    FilePath(const char* filePath_);
    FilePath GetWorkspace() const;
    FilePath GetBackupFilePath() const;
    string GetTimeStamp() const;
	FilePath Mirror(string module);
    void Append(string pathItems);
    void SetFilePath(string path);
    bool SetWriteable() const;
    bool SetReadOnly() const;
    bool IsWriteable() const;
    bool IsDirectory() const;
	bool IsDirectoryEmpty() const;
	bool SetCurrentDirectory() const;
    bool Exists() const;
    bool IsControlled() const;
	bool IsControlledDirectory() const;
	bool IsEmpty() const;
	bool IsZeroVersion() const;
    string GetEntryTimeStamp() const;
    string GetBaseRevision() const;
	vector<FilePath> GetEntriesFiles() const;
	string GetProjectRoot() const;
	bool IsValid() const;
    const char* FilePathString() const;
    string FileName() const;
    string DirectoryName() const;
	FilePath ParentDirectory() const;
    bool IsDirSeparator(int i) const;
	bool operator< (const FilePath& otherPath) const;
	bool operator== (const FilePath& otherPath) const;
    bool CreateDirectory() const;
	static FilePath GetCurrentDirectoryFilePath();
	static FilePath GetTempDirectoryFilePath();
	bool IsTimeStampSameAsEntry() const;
	enum ForceCase { off=0, lower, upper };
	string GetRepository(ForceCase forceCase=off) const;
private:
    FilePath EntriesFilePath() const;
	Entries& Entries() const;
    string filePath;
	friend ostream& operator<<(ostream& str, const FilePath& filePath);
};

#pragma warning (disable:4786)
typedef vector<FilePath> FilePathVector;
#pragma warning (disable:4786)
typedef set<FilePath> FilePathSet;

#endif // _FilePath_h