#include "stdafx.h"
#include "FilesMonitor.h"
#include <set>
#include <map>
#include "IglooDialogs.h"
#include <fstream>

#pragma warning (disable:4786)
typedef vector<HANDLE> HandleVector;
typedef pair<HANDLE*, int> HandleArray;
typedef std::set<FilePath> DirectorySet;
typedef std::map<HANDLE, FilePath> DirectoryMap;


class DirectoriesMonitor
{
public:
	DirectoriesMonitor(FilesMonitor& filesMonitor_);
	~DirectoriesMonitor();
	UINT Run();
	void Close();
	void Signal();
	bool IsFull();
	void Monitor(FilePath directory);
	void SetEnabled(bool enabled);

private:
	void AddHandle(HANDLE handle);
	int CountHandles();
	HandleArray CreateHandleArray(HandleArray handleArray);
	CriticalSection lock;
	CriticalSection mapLock;
	HandleVector handles;
	HANDLE signalHandle;
	HANDLE closeHandle;
	FilesMonitor& filesMonitor;
	DirectorySet directorySet;
	DirectoryMap directoryMap;
	DirectorySet processSet;
	bool closed;
	bool run;
	bool enabled;
};

static UINT ThreadStart( LPVOID pParam )
{
    DirectoriesMonitor* pObject = (DirectoriesMonitor*)pParam;

	UINT result= pObject->Run();
//	delete pObject;
	return result;
}

DirectoriesMonitor::DirectoriesMonitor(FilesMonitor& filesMonitor_) :
filesMonitor(filesMonitor_), closed(false), run(false), enabled(true)
{
	signalHandle = CreateEvent( 
        NULL,         // no security attributes
        FALSE,         // manual-reset event
        FALSE,         // initial state is signaled
        NULL  // object name
        ); 
	AddHandle(signalHandle);
}

DirectoriesMonitor::~DirectoriesMonitor()
{
//	dout.close();
}


void DirectoriesMonitor::Close()
{
	if (!run)
	{	
		closed=true;
		return;
	}
	closeHandle= CreateEvent( 
        NULL,         // no security attributes
        TRUE,         // manual-reset event
        FALSE,         // initial state is signaled
        NULL  // object name
        ); 
	closed=true;
	Signal();
	WaitForSingleObject(closeHandle, INFINITE);
	CloseHandle(closeHandle);
}

void DirectoriesMonitor::Signal()
{
	SetEvent(signalHandle);
}

bool DirectoriesMonitor::IsFull()
{
	return (CountHandles()+2 >= MAXIMUM_WAIT_OBJECTS);
}

void DirectoriesMonitor::SetEnabled(bool enabled_)
{
	if (enabled!=enabled_)
	{
		enabled=enabled_;
		Signal();
	}
}

void DirectoriesMonitor::Monitor(FilePath directory)
{
    if (!directory.IsDirectory()) return;
	if (CountHandles()==MAXIMUM_WAIT_OBJECTS)
	{
		// We've run out of handles;
		return;
	}
	directory=directory.FilePathString();
	DirectorySet::iterator iterator=directorySet.find(directory);
	if (iterator==directorySet.end())
	{
		directorySet.insert(directory);
		HANDLE handle=FindFirstChangeNotification(directory.FilePathString(),
			FALSE, FILE_NOTIFY_CHANGE_LAST_WRITE);
        if (handle == INVALID_HANDLE_VALUE)
        {
//            dout << "Invalid Handle\n";
			return;
        }
        else 
        {
            mapLock.Enter();
            int size=directoryMap.size();
			directoryMap.insert(DirectoryMap::value_type(handle, directory));
			mapLock.Leave();
			AddHandle(handle);
			Signal();
        }
	}
	return;
}

void DirectoriesMonitor::AddHandle(HANDLE handle)
{
	lock.Enter();
	handles.push_back(handle);
	lock.Leave();
}

int DirectoriesMonitor::CountHandles()
{
	int size=0;
	lock.Enter();
	size=handles.size();
	lock.Leave();
	return size;
}


HandleArray DirectoriesMonitor::CreateHandleArray(HandleArray handleArray)
{
	lock.Enter();
	int size=handles.size();
	if (handleArray.second!=size)
	{
		delete [] handleArray.first;
		handleArray.first=new HANDLE[handles.size()];
		handleArray.second=size;
		for (int i=0; i<handles.size(); i++)
		{
			handleArray.first[i]=handles[i];
		}
	}
	lock.Leave();
	return handleArray;
}
	
UINT DirectoriesMonitor::Run()
{
	run=true;
	HandleArray handleArray=CreateHandleArray(HandleArray(0, 0));
	DWORD timeout=INFINITE; //WAIT_TIMEOUT
	while (!closed && run)
	{
		DWORD result=0;
		bool waited=false;
		if (enabled)
		{
			result=WaitForMultipleObjects(
			handleArray.second,             // number of handles in array
			handleArray.first,  // object-handle array
			FALSE,            // wait option
			timeout      // time-out interval
			);
			waited=true;
		}
		while (!enabled && !closed)
		{
			WaitForSingleObject(handleArray.first[0], INFINITE);
		}
		if (closed) break;
		if (!waited) continue;
		if (result==WAIT_OBJECT_0)
		{
//		    dout << "Signal\n";
		}
		if (result==WAIT_ABANDONED_0)
		{
	        run=false;
		}
        if (result==WAIT_FAILED)
        {
			run=false;
			/*LPVOID lpMsgBuf;
			FormatMessage( 
				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				FORMAT_MESSAGE_FROM_SYSTEM | 
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				GetLastError(),
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL 
				);
            dout << "Failed " << lpMsgBuf << endl;// 
			LocalFree( lpMsgBuf );*/
        }
		if (result>WAIT_OBJECT_0 && result< WAIT_OBJECT_0 + handleArray.second)
		{
			int index=result-WAIT_OBJECT_0;
			HANDLE handleFound=handleArray.first[index];
			mapLock.Enter();
			FilePath directory=directoryMap[handleFound].FilePathString();
		//		dout << directory << endl;
			mapLock.Leave();
			processSet.insert(directory);
			FindNextChangeNotification(handleFound);
			timeout=200;
		}
		if (result>WAIT_ABANDONED_0  && result< WAIT_ABANDONED_0  + handleArray.second)
		{
			run=false;
			/*int index=result-WAIT_OBJECT_0;
			HANDLE handleFound=handleArray.first[index];
			mapLock.Enter();
			FilePath directory=directoryMap[handleFound].FilePathString();
				dout << "Abandon " << directory << endl;
			mapLock.Leave();*/
        }
        if (result==WAIT_TIMEOUT)
		{
			for (DirectorySet::iterator iterator=processSet.begin();
			iterator!=processSet.end(); iterator++)
			{
				filesMonitor.ProcessDirectory(*iterator);
			}
			processSet.erase(processSet.begin(), processSet.end());
			timeout=INFINITE;
		}
		handleArray=CreateHandleArray(handleArray);
	}

	CloseHandle(handleArray.first[0]);
	lock.Enter();
	for (int i=1; i<handleArray.second; i++)
	{
		FindCloseChangeNotification(handleArray.first[i]);
	}
	delete [] handleArray.first;
	lock.Leave();
	if (closed) SetEvent(closeHandle);
	return 0;
}

FilesMonitor::FilesMonitor()
{
}

void FilesMonitor::Monitor(FilePath filePath, Eskimo::FileStatus status)
{
	filePath=filePath.FilePathString();
	int size=directoriesMonitors.size();
	DirectoriesMonitor* directoriesMonitor= size==0?0:directoriesMonitors[size-1];
	if (directoriesMonitor==0 || directoriesMonitor->IsFull())
	{
		directoriesMonitor=new DirectoriesMonitor(*this);
		AFX_MANAGE_STATE(AfxGetStaticModuleState( ));
		AfxBeginThread(ThreadStart, directoriesMonitor);
		directoriesMonitors.push_back(directoriesMonitor);
	}
	filesLock.Enter();
	statusMap[filePath]=status;
	string directory=filePath.DirectoryName();
	FilePathSet* files=0;
	FilesMap::iterator iterator=filesMap.find(directory);
	if (iterator==filesMap.end())
	{
		files=new FilePathSet();
		filesMap.insert(FilesMap::value_type(directory, files));
	}
	else
	{
		files=(*iterator).second;
	}
	FilePathSet::iterator iterator2=files->find(filePath);
	if (iterator2==files->end()) files->insert(filePath);
	FilePath cvsDir=directory;
	cvsDir.Append("CVS");
	cvsDirectories.insert(cvsDir);
	filesLock.Leave();
	directoriesMonitor->Monitor(directory);
	directoriesMonitor->Monitor(cvsDir);

}

void FilesMonitor::ProcessDirectory(FilePath directory)
{
	directory=directory.FilePathString();
	filesLock.Enter();
	FilePathSet::iterator iterator=cvsDirectories.find(directory);
	if (iterator!=cvsDirectories.end())
	{
		FilesMap::iterator iterator=filesMap.find(directory.ParentDirectory());
		if (iterator!=filesMap.end())
		{
			ProcessFiles(*((*iterator).second));
		}
	}
	{
		FilesMap::iterator iterator=filesMap.find(directory);
		if (iterator!=filesMap.end())
		{
			ProcessFiles(*((*iterator).second));
		}
	}
	filesLock.Leave();
}

void FilesMonitor::ProcessFiles(const FilePathSet& files)
{
	for (FilePathSet::const_iterator iterator=files.begin();
	iterator!=files.end(); iterator++)
	{
		FilePath file = *iterator;
		Eskimo::FileStatus status=Eskimo::DoQuery(file);
		if (status!=statusMap[file])
		{
			statusMap[file]=status;
			
			lock.Enter();
			eventSet.insert(file);
			lock.Leave();
//			eskimo->Dialogs()->NotifyIDE();
		}
		Sleep(20);
	}
}


Eskimo::FileChanged FilesMonitor::NextFileChanged()
{
	Eskimo::FileChanged fileChanged;
	lock.Enter();
	FilePathSet::iterator iterator=eventSet.begin();
	fileChanged.file=*iterator;
	fileChanged.status=statusMap[fileChanged.file];
	eventSet.erase(iterator);
	lock.Leave();
	return fileChanged;
}

int FilesMonitor::FilesChanged()
{
	int changed=0;
	lock.Enter();
	changed=eventSet.size();
	lock.Leave();
	return changed;
}


FilesMonitor::~FilesMonitor()
{
	for (DirectoriesMonitors::iterator iterator=directoriesMonitors.begin();
	iterator!=directoriesMonitors.end(); iterator++)
	{
		DirectoriesMonitor* directoriesMonitor=(*iterator);
		directoriesMonitor->Close();
		delete directoriesMonitor;
	}
	for (FilesMap::const_iterator iterator2=filesMap.begin();
	iterator2!=filesMap.end(); iterator2++)
	{
		FilePathSet* fileSet = (*iterator2).second;
		delete fileSet;
	}
}

void FilesMonitor::SetEnabled(bool enabled)
{
	for (DirectoriesMonitors::iterator iterator=directoriesMonitors.begin();
	iterator!=directoriesMonitors.end(); iterator++)
	{
		DirectoriesMonitor* directoriesMonitor=(*iterator);
		directoriesMonitor->SetEnabled(enabled);
	}	
}

