#include "stdafx.h"

#include "GoCvs.h"
#include <AppGlue.h>
#include <CvsArgs.h>
#include "Eskimo.h"
#include "IglooDialogs.h"
#include "ParentDisable.h"

using namespace std;

class CNullConsole : public CCvsConsole
{
public:
	CNullConsole(GoCvs* cvs_) : cvs(cvs_), loose(false) , useWinCvslib(false)
	{
		::InitializeCriticalSection(&lock);
	}
	virtual long Out(char *txt, long len)
	{
        char *s=new char[len+1];
        strncpy(s, txt, len);
        s[len]=0;
		last=s;
        Output(s);
		loose = s[len-1]!='\n';
        delete [] s;
		return len;
	}
	virtual long Err(char *txt, long len)
	{
        char *s=new char[len+1];
        strncpy(s, txt, len);
        s[len]=0;
		string str(s);
        if (len==0) {delete [] s; return len;}
        if (str.find("Garbage")!=str.npos) {delete [] s; return len;}
        Output(s);
		if (str.find("scheduling file")==str.npos &&
			str.find("cvs commit")==str.npos)
		{
			cvs->ErrorLogged();
		}
		loose = s[len-1]!='\n';
        delete [] s;
		return len;
	}
	virtual void Flush()
	{
		if (loose) Output("\n");
		loose = false;
	}
	virtual char* GetPassword()
	{
		return cvs->GetPassword();
	}
	virtual long In(char *txt, long len)
	{
		Output("Asking for input\n");
//		cvs->Question(last);
		return 0;
	}
	virtual bool YesNo()
	{
		if (last.length()>0 && last[last.length()-1]=='\n') last=last.substr(0, last.length());
		bool result=cvs->YesNo(last);
		if (result) Output("yes\n");
		else Output("no\n");
		return result;
	}
    virtual bool ShouldStop()
    {
        SetEvent(handle);
        return cvs->ShouldStop();
    }
    HANDLE handle;
	bool useWinCvslib;
    bool done;
	void AddToOut(char* s)
	{
		::EnterCriticalSection(&lock);
		toOut += s;
		::LeaveCriticalSection(&lock);
	}
	string GetAndClearOut()
	{
		::EnterCriticalSection(&lock);
		string copyOut(toOut.c_str());
		toOut=string();
		::LeaveCriticalSection(&lock);
		return copyOut;
	}

protected:
	virtual void Output(char* s)
	{
	}
	GoCvs* cvs;
	bool loose;
    string last;
private:
	CRITICAL_SECTION lock;
	string toOut;
};

class CStreamConsole : public CNullConsole
{
public:
	CStreamConsole(GoCvs* cvs_, ostream *stream) : CNullConsole(cvs_), outStream(*stream) {}
protected:
	virtual void Output(char* s)
	{
		AddToOut(s);
//		outStream << s;
        SetEvent(handle);
	}
private:
	ostream &outStream;
};

GoCvs::GoCvs() : eskimo(0)
{
	password[0]=0;
}

void GoCvs::SetPassword(string password_)
{
	if (password_.length()<500)
	{
		strcpy(password, password_.c_str());
	}
	else
	{
		password[0]=0;
	}
}

char* GoCvs::GetPassword()
{
	return password;
}

bool GoCvs::YesNo(string question)
{
	return Dialogs()->YesNo(question);
}

void GoCvs::ErrorLogged()
{
	eskimo->ErrorLogged();
}

void GoCvs::SetEskimo(Eskimo* eskimo_)
{
	eskimo=eskimo_;
}

static bool IsNtServer(string cvsroot)
{
	string ntserver=":ntserver:";
/*	if (root.length()>=ntserver.length())
	{
		if (strncmp(root.c_str(), ntserver.c_str(), ntserver.length())==0) return true;
	}
	return false;*/
	return cvsroot.find(ntserver)!=string.npos;
}

/* Tries to determine if we are using an ntserver for logins to CVS
	If the dir is controlled then we check the CVS/Root file for :ntserver:.
	If the dir is not controlled then we check if there is a root variable
	on the command line. If not then we check the CVSROOT variable */
bool GoCvs::IsNtServer(FilePath dir, const CvsArgs & args)
{
	string root=dir.GetProjectRoot();
	if (root.length()==0)
	{
		for (int i=0; i<args.Argc(); i++)
		{
			string arg=args.Argv()[i];
			if (arg.find("-d")!=string.npos)
			{
				if (::IsNtServer(arg)) return true;
				if (i+1 < args.Argc() && ::IsNtServer(args.Argv()[i+1])) return true;
			}
		}
		char *cvsroot=getenv("CVSROOT");
		if (cvsroot!=NULL) root=cvsroot;
	}
	return ::IsNtServer(root);
}

class LaunchThread : public CWinThread
{
public:
	LaunchThread() : result(0)
	{
	}
	LaunchThread(GoCvs* cvs_, bool wincvslib_, string path_, CvsArgs args_,
	CNullConsole * console_, int *result_) : wincvslib(wincvslib_), cvs(cvs_), args(args_),
	console(console_), result(result_)
	{
		path=path_.c_str();
	}
	virtual BOOL InitInstance( )
	{
		*result=launchCVS(wincvslib, path.c_str(), args.Argc(), args.Argv(), console);
//		::Sleep(5000);
		return 0;
	}
	virtual int ExitInstance()
	{
        console->done=true;
		SetEvent(console->handle);
		return 1;
	}
	bool wincvslib;
	string path;
	CvsArgs args;
    GoCvs* cvs;
	CNullConsole *console;
	int *result;
	DECLARE_DYNCREATE(LaunchThread)
};

IMPLEMENT_DYNCREATE(LaunchThread,CWinThread)


IglooDialogs* GoCvs::Dialogs()
{
	return eskimo->Dialogs();
}


int GoCvs::LaunchInThread(bool wincvslib, string path, CvsArgs args,
	CNullConsole * console, ostream *stream, bool process)
{
	if (!process)
	{
		return launchCVS(wincvslib, path.c_str(), args.Argc(), args.Argv(), console);
	}
	ParentDisable parentDisable(eskimo);
    console->done=false;
	console->handle = CreateEvent( 
        NULL,         // no security attributes
        TRUE,         // manual-reset event
        FALSE,         // initial state is signaled
        NULL  // object name
        ); 
	int result;
	LaunchThread* thread= new LaunchThread(this, wincvslib, path,
		args, console, &result);
	thread->CreateThread();
    while (!console->done)
    {
        WaitForSingleObject(console->handle, process?500:INFINITE);
        ResetEvent(console->handle);
		string outputString=console->GetAndClearOut();
		if (stream && outputString.length()>0)
		{
			*stream << outputString;
		}
        if (process) eskimo->ProcessSomeMessages();
    }
	CloseHandle(console->handle);
    console->handle=0;
	return result;
}

bool GoCvs::ShouldStop()
{
    return Dialogs()->ShouldStop();
}

int GoCvs::Launch(FilePath dir, const CvsArgs & args, ostream *stream)
{
	CStreamConsole myconsole(this, stream);
	int result= LaunchInThread(!IsNtServer(dir, args), dir.FilePathString(),
		args, &myconsole, stream, true);
	return result;
//	return launchCVS(!IsNtServer(dir, args), dir.filePathString(), args.Argc(), args.Argv(), &myconsole);
}

int GoCvs::Launch(FilePath dir, const CvsArgs & args)
{
	CNullConsole myconsole(this);
	int result= LaunchInThread(!IsNtServer(dir, args), dir.FilePathString(),
		args, &myconsole, 0, false);
	return result;
}


