#include "stdafx.h"
#include "UserSettings.h"

UserSettings::UserSettings() : workIndependently(false),
workOffline(false), warnOnEdit(true), mirror(true), readOnlyFiles(true),
mirrorPath("c:\\mirror")
{
//	diffExes["regular"]="";
//	diffExes["araxis"]="";
    Load();
}

void UserSettings::SetWarnOnEdit(bool value)
{
	warnOnEdit=value;
}

bool UserSettings::WarnOnEdit()
{
	return warnOnEdit;
}

bool UserSettings::WorkIndependently()
{
    return workIndependently;
}

void UserSettings::SetDiffType(DiffType type)
{
	switch (type)
	{
	case CVS:
		diffType="cvs";
		break;
	case REGULAR:
		diffType="regular";
		break;
	case ARAXIS:
		diffType="araxis";
		break;
	case ARAXIS_PRO:
		diffType="araxisPro";
		break;
	}
}

UserSettings::DiffType UserSettings::GetDiffType()
{
	if (diffType.size())
	{
		if (diffType=="regular")
		{
			return REGULAR;
		}
		if (diffType=="cvs")
		{
			return CVS;
		}
		if (diffType=="araxis")
		{
			return ARAXIS;
		}
		return ARAXIS_PRO;
	}
	return CVS;
}

bool UserSettings::WorkOffline()
{
    return workOffline;
}

string UserSettings::GetDiffExe()
{
/*	StringMap::const_iterator iterator=diffExes.find(type);
	if (iterator==diffExes.end())
	{
		return "";
	}
	return (*iterator).second;*/
	return regularExe;
}

void UserSettings::SetDiffExe(string exe)
{
	regularExe=exe;
}


void UserSettings::SetWorkIndependently(bool value)
{
    workIndependently=value;
}

void UserSettings::SetWorkOffline(bool value)
{
    workOffline=value;
}

bool UserSettings::AutoCommit()
{
    return !workOffline && !workIndependently;
}

bool UserSettings::ReadOnlyFiles()
{
	return readOnlyFiles;
}

bool UserSettings::Mirror()
{	
	return mirror;
}

FilePath UserSettings::MirrorPath()
{
	return mirrorPath;
}


class RegistrySettings
{
public:
    RegistrySettings()
    {
        DWORD disposition;
    RegCreateKeyEx(HKEY_CURRENT_USER, "SOFTWARE\\Jalindi\\Igloo", 0, NULL,
        REG_OPTION_NON_VOLATILE, KEY_QUERY_VALUE | KEY_SET_VALUE, NULL, &hkey, &disposition);
    }
    ~RegistrySettings()
    {
        RegCloseKey(hkey);
    }
    void GetSetBoolean(const char* name, bool& boolean, UserSettings::GetSet getSet)
	{
		if (getSet==UserSettings::GET)
		{
			DWORD value;
			DWORD length=sizeof(DWORD);
			DWORD type;
			RegQueryValueEx(hkey, name, NULL, &type, (BYTE*)&value, &length);
			if (length>0 && type==REG_DWORD)
			{
				boolean=value==1;
			}
		}
		else
		{
			DWORD value=boolean;
			RegSetValueEx(hkey, name, 0, REG_DWORD , (BYTE*)&value, sizeof(DWORD));
		}
	}

    void GetSetString(const char* name, string& stringData, UserSettings::GetSet getSet)
	{
		if (getSet==UserSettings::GET)
		{
			DWORD data[1000];
			DWORD length=1000;
			DWORD type;
			RegQueryValueEx(hkey, name, NULL, &type, (BYTE*)data, &length);
			if (length>0 && type==REG_SZ)
			{
				stringData=(char*)data;
			}
		}
		else
		{
			RegSetValueEx(hkey, name, 0, REG_SZ , (BYTE*)(stringData.c_str()), stringData.length()+1);
		}
	}
private:
    HKEY hkey;
};

void UserSettings::Load()
{
    LoadSave(GET);
}


void UserSettings::Save()
{
    LoadSave(SET);
}

void UserSettings::LoadSave(GetSet getSet)
{
    RegistrySettings registry;
    registry.GetSetBoolean("WorkOffline", workOffline, getSet);
    registry.GetSetBoolean("WorkIndependently", workIndependently, getSet);
    registry.GetSetBoolean("WarnOnEdit", warnOnEdit, getSet);
	registry.GetSetString("DiffType", diffType, getSet);
	registry.GetSetString("RegularDiffExe", regularExe, getSet);
/*	for (StringMap::iterator iterator=diffExes.begin();
	iterator!=diffExes.end(); ++iterator)
	{
		string key=string("Diff")+(*iterator).first;
		registry.GetSetString(key.c_str(), (*iterator).second, getSet);
	}*/
	for (int i=0; i<STRING_MEMORYS; i++)
	{
		char setting[100];
		strcpy(setting, "Root_A");
		int last=strlen(setting)-1;
		char c=setting[last];
		char d=((char)(i)+'A');
		setting[last]=d;
		if (getSet==GET || roots[i].length()>0) registry.GetSetString(setting, roots[i], getSet);
		strcpy(setting, "Vendor_A");
		setting[strlen(setting)-1]=((char)(i)+'A');
		if (getSet==GET || vendors[i].length()>0) registry.GetSetString(setting, vendors[i], getSet);
	}
}

void UserSettings::SetRoot(string root)
{
	if (root.length()==0) return;
	for (int i=0; i<STRING_MEMORYS; i++)
	{
		if (roots[i].compare(root) == 0)
		{
			for (; i<STRING_MEMORYS-1; i++)
			{
				roots[i]=roots[i+1];
			}
			roots[i]="";
			break;
		}
	}
	for (i=STRING_MEMORYS-1; i>0; i--)
	{
		roots[i]=roots[i-1];
	}
	roots[0]=root;
}

void UserSettings::SetVendor(string vendor)
{
	if (vendor.length()==0) return;
	for (int i=0; i<STRING_MEMORYS; i++)
	{
		if (vendors[i].compare(vendor) == 0)
		{
			for (; i<STRING_MEMORYS-1; i++)
			{
				vendors[i]=vendors[i+1];
			}
			vendors[i]="";
			break;
		}
	}
	for (i=STRING_MEMORYS-1; i>0; i--)
	{
		vendors[i]=vendors[i-1];
	}
	vendors[0]=vendor;
}

string UserSettings::GetRoot(int index)
{
	return roots[index];
}

string UserSettings::GetVendor(int index)
{
	return vendors[index];
}

int UserSettings::GetNumberRoots()
{
	for (int i=0; i<STRING_MEMORYS; i++)
	{
		if (roots[i].length()==0) return i;
	}
	return i;
}

int UserSettings::GetNumberVendors()
{
	for (int i=0; i<STRING_MEMORYS; i++)
	{
		if (vendors[i].length()==0) return i;
	}
	return i;
}